# Plugin written by Tim Taylor, timtaylor3@yahoo.com
from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Returns the user names with sids"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        path = r"Microsoft\Windows NT\CurrentVersion\ProfileList"
        key = self.open_key(path)
        if not key:
            return

        sid_list = list()

        for v in key.subkeys():
            sid_list.append(v.name())

        for sid in sid_list:
            sid_path = path + "\\" + sid
            key2 = self.open_key(sid_path)

            if not key2:
                continue

            for entry in key2.values():
                if entry.name() == "ProfileImagePath":
                    res = PluginResult(key=key, value=entry)
                    user_name = entry.value().split("\\")[-1]
                    res.custom["value"] = "{}:\t{}".format(user_name, key2.name())
                    yield res

    def display_human(self, result):
        print(result.custom["value"])

    def display_machine(self, result):
        print(mactime(name=result.custom["value"], mtime=result.mtime))



import pytest
from Registry.Registry import RegDWord, RegSZ

from regrippy.plugins.sysinternals import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("Software\\Sysinternals")
    reg = RegistryMock("SOFTWARE", "software", key.root())

    # Accepted
    sysmon = RegistryKeyMock("Sysmon", key)
    key.add_child(sysmon)
    sysmon_eula = RegistryValueMock("EulaAccepted", 1, RegDWord)
    sysmon.add_value(sysmon_eula)

    # Rejected
    procdump = RegistryKeyMock("Procdump", key)
    key.add_child(procdump)
    procdump_eula = RegistryValueMock("EulaAccepted", 0, RegDWord)
    procdump.add_value(procdump_eula)

    # No EulaAccepted
    procexp = RegistryKeyMock("Procexp", key)
    key.add_child(procexp)

    return reg


def test_sysinternals(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SOFTWARE", "-")

    results = list(p.run())

    assert len(results) == 3, "There should be 3 results"

    assert results[0].key_name == "Sysmon", "First result should be Sysmon"
    assert results[0].value_data == 1, "Sysmon should be 1 (accepted)"

    assert results[1].key_name == "Procdump", "Second result should be Procdump"
    assert results[1].value_data == 0, "Procdump should be 0 (refused)"

    assert results[2].key_name == "Procexp", "Third result should be Procexp"
    assert results[2].value_data == None, "Procexp shouldn't have a 'value'"



import pytest
from Registry.Registry import RegBin

from regrippy.plugins.lastshutdown import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("ControlSet001\\Control\\Windows")
    reg = RegistryMock("SYSTEM", "system", key.root())
    reg.set_ccs(1)

    val = RegistryValueMock("ShutdownTime", b"\xe40\xa5!\xed\xf7\xd3\x01", RegBin)
    key.add_value(val)

    return reg


def test_lastshutdown(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SYSTEM", "-")

    results = list(p.run())
    assert len(results) == 1, "There should be a single result"
    assert (
        results[0].custom["LastShutdownTime"] == "2018-05-30T08:06:36.766026Z"
    ), "It should have returned 2018-05-30T08:06:36.766026Z"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.keyboard import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("Keyboard Layout")
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    preload = RegistryKeyMock("Preload", key)
    key.add_child(preload)

    substitutes = RegistryKeyMock("Substitutes", key)
    key.add_child(substitutes)

    french_layout_standard = RegistryValueMock("1", "0000040c", RegSZ)
    preload.add_value(french_layout_standard)

    french_canadian = RegistryValueMock("2", "d000040c", RegSZ)
    french_canadian_subs = RegistryValueMock("d000040c", "00001009", RegSZ)
    preload.add_value(french_canadian)
    substitutes.add_value(french_canadian_subs)

    return reg


def test_keyboard(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())
    assert len(results) == 2, "There should be two results"
    assert any(
        [x.custom["substitute"] is not None for x in results]
    ), "One of the results should have a substitution"
    assert any(
        [x.custom["substitute"] is None for x in results]
    ), "One of the results should NOT have a substitution"



#!/usr/bin/env python3

import argparse
import logging
import os
import sys

import pkg_resources
from Registry import Registry

logging.basicConfig()
l = logging.getLogger("regrippy")
l.setLevel("ERROR")


def first(*args):
    for arg in args:
        if arg:
            return arg
    return None


def find_file_nocase(root, file_name):
    for file in os.listdir(root):
        if file.lower() == file_name.lower():
            return file
    return None


def find_path_nocase(root_path, rest_list):
    full_path_parts = []
    for part in rest_list:
        actual_name = find_file_nocase(os.path.join(root_path, *full_path_parts), part)
        if not actual_name:
            return None
        full_path_parts.append(actual_name)

    return os.path.join(root_path, *full_path_parts)


def get_hive_paths(args, hive_name):
    # use REG_ROOT to set root path of hive
    args.root = args.root or os.getenv("REG_ROOT")

    if (
        hive_name.lower() in ["all", "ntuser.dat", "usrclass.dat"]
        and args.all_user_hives
        and not args.root
    ):
        print("Error: --all-user-hives requires --root", file=sys.stderr)
        sys.exit(3)

    # Basic cases
    if hive_name.lower() == "system":
        path = first(
            args.system,
            find_path_nocase(args.root, ["windows", "system32", "config", "system"])
            if args.root
            else None,
            os.getenv("REG_SYSTEM"),
        )
        result = [path] if path else None
        if args.backups and path:
            parent = os.path.dirname(path)
            regback = find_path_nocase(parent, ["regback", "system"])
            if regback:
                result.append(regback)
        return result
    elif hive_name.lower() == "software":
        path = first(
            args.software,
            find_path_nocase(args.root, ["windows", "system32", "config", "software"])
            if args.root
            else None,
            os.getenv("REG_SOFTWARE"),
        )
        result = [path] if path else None
        if args.backups and path:
            parent = os.path.dirname(path)
            regback = find_path_nocase(parent, ["regback", "software"])
            if regback:
                result.append(regback)
        return result
    elif hive_name.lower() == "sam":
        path = first(
            args.sam,
            find_path_nocase(args.root, ["windows", "system32", "config", "sam"])
            if args.root
            else None,
            os.getenv("REG_SAM"),
        )
        result = [path] if path else None
        if args.backups and path:
            parent = os.path.dirname(path)
            regback = find_path_nocase(parent, ["regback", "sam"])
            if regback:
                result.append(regback)
        return result
    elif hive_name.lower() == "ntuser.dat" and not args.all_user_hives:
        path = first(args.ntuser, os.getenv("REG_NTUSER"))
        result = [path] if path else None
        if args.backups and path:
            parent = os.path.dirname(path)
            regback = find_path_nocase(parent, ["ntuser.dat.old"])
            if regback:
                result.append(regback)
        return result
    elif hive_name.lower() == "usrclass.dat" and not args.all_user_hives:
        path = first(args.usrclass, os.getenv("REG_USRCLASS"))
        result = [path] if path else None
        if args.backups and path:
            parent = os.path.dirname(path)
            regback = find_path_nocase(parent, ["usrclass.dat.old"])
            if regback:
                result.append(regback)
        return result

    # All user hives
    elif hive_name.lower() in ["ntuser.dat", "usrclass.dat"] and args.all_user_hives:
        hive_paths = []
        users_folder = find_path_nocase(args.root, ["users"])
        if users_folder is None:
            # Not found, let's try "Documents and Settings"
            users_folder = find_path_nocase(args.root, ["Documents And Settings"])

        if users_folder is None:
            # Still nothing, we didn't find the Users folder, crash&burn
            raise RuntimeError("Could not find the Users folder")

        for user_dir in os.listdir(users_folder):
            current_user_dir = os.path.join(users_folder, user_dir)
            if not os.path.isdir(current_user_dir):
                continue
            if hive_name.lower() == "ntuser.dat":
                path = find_path_nocase(current_user_dir, ["ntuser.dat"])
                if path is not None:
                    hive_paths.append(path)

                    if args.backups:
                        backup = find_path_nocase(current_user_dir, ["ntuser.dat.old"])
                        if backup:
                            hive_paths.append(backup)
            else:
                path = find_path_nocase(
                    current_user_dir,
                    ["appdata", "local", "microsoft", "windows", "usrclass.dat"],
                )
                if path is not None:
                    hive_paths.append(path)

                    if args.backups:
                        backup = find_path_nocase(
                            current_user_dir,
                            [
                                "appdata",
                                "local",
                                "microsoft",
                                "windows",
                                "usrclass.dat.old",
                            ],
                        )
                        if backup:
                            hive_paths.append(backup)

        return hive_paths

    elif hive_name.lower() == "all":
        hive_paths = []
        for hive in ["system", "software", "sam", "ntuser.dat", "usrclass.dat"]:
            paths = get_hive_paths(args, hive)
            if paths:
                hive_paths.extend(paths)
        return hive_paths

    return None


def list_plugins():
    for ep in pkg_resources.iter_entry_points(group="plugins"):
        plugin = ep.load()
        print(f"- {ep.name}({plugin.__REGHIVE__}): {plugin.__doc__}")


def load_plugin(plugin_name):
    for ep in pkg_resources.iter_entry_points(group="plugins"):
        if ep.name == plugin_name:
            return ep.load()

    raise ValueError(f"No such plugin: {plugin_name}")


def main():
    if os.path.basename(sys.argv[0]).lower() != "regrip.py":
        # Issue #5: allow selecting plugins based on argv[0]
        plugin_name = os.path.basename(sys.argv[0])

        # Allow the symlink to be called reg_pluginname to reduce risk of collision
        if plugin_name.startswith("reg_"):
            plugin_name = plugin_name[len("reg_") :]
    else:
        plugin_name = None

    parser = argparse.ArgumentParser(
        description="Extract information from Windows Registry hives"
    )

    parser.add_argument(
        "--system",
        "-y",
        help="Path to the SYSTEM hive. Overrides --root and the REG_SYSTEM environment variable",
        type=str,
        default="",
    )
    parser.add_argument(
        "--software",
        "-o",
        help="Path to the SOFTWARE hive. Overrides --root and the REG_SOFTWARE environment variable",
        type=str,
        default="",
    )
    parser.add_argument(
        "--sam",
        "-a",
        help="Path to the SAM hive. Overrides --root and the REG_SAM environment variable",
        type=str,
        default="",
    )
    parser.add_argument(
        "--ntuser",
        "-n",
        help="Path to the NTUSER.DAT hive. Overrides the REG_NTUSER environment variable",
        type=str,
        default="",
    )
    parser.add_argument(
        "--usrclass",
        "-u",
        help="Path to the UsrClass.DAT hive. Overrides the REG_USRCLASS environment variable",
        type=str,
        default="",
    )
    parser.add_argument(
        "--root",
        "-r",
        help="Path to the C: folder. Overrides the REG_ROOT environment variable",
        type=str,
        default="",
    )
    parser.add_argument(
        "--all-user-hives",
        help="Work on all NTUSER.DAT and USRCLASS.DAT hives if required. Requires --root. Overrides --ntuser and --usrclass.",
        action="store_true",
    )
    parser.add_argument(
        "--backups",
        action="store_true",
        help="Run the plugin on backup registry hives as well (does not work for hives loaded from stdin)",
    )
    parser.add_argument("--verbose", "-v", help="Be more verbose", action="store_true")
    parser.add_argument(
        "--bodyfile", "-b", help="Force output in Bodyfile format", action="store_true"
    )
    parser.add_argument(
        "--list", "-l", help="List available plugins", action="store_true"
    )

    if not plugin_name:
        parser.add_argument("plugin_name", help="Name of the plugin to run", type=str)

    if "--list" in sys.argv or "-l" in sys.argv:
        list_plugins()
        return

    args = parser.parse_args()
    if not plugin_name:
        plugin_name = args.plugin_name

    if args.verbose:
        l.setLevel("DEBUG")

    plugin = load_plugin(plugin_name)

    if type(plugin.__REGHIVE__) == str:
        hive_names = [plugin.__REGHIVE__]
    else:
        hive_names = plugin.__REGHIVE__

    for hive_name in hive_names:
        hive_paths = get_hive_paths(args, hive_name)
        if not hive_paths:
            print("[!] Hive not found:", hive_name, file=sys.stderr)
            continue

        for hive_path in hive_paths:
            if hive_path == "-":
                # Special case: read hive from stdin
                reg = Registry.Registry(sys.stdin.buffer)
            else:
                reg = Registry.Registry(hive_path)

            p = plugin(reg, l, hive_name, hive_path)
            results = list(p.run())

            if results:
                for result in results:
                    if args.bodyfile:
                        p.display_machine(result)
                    else:
                        if hive_name == "NTUSER.DAT":
                            print(f"[.] User: {p.guess_username()}")
                        p.display_human(result)


if __name__ == "__main__":
    main()



import pkg_resources
from Registry.Registry import (RegBigEndian, RegBin, RegDWord, RegExpandSZ,
                               RegFullResourceDescriptor,
                               RegistryKeyNotFoundException,
                               RegistryValueNotFoundException, RegLink,
                               RegMultiSZ, RegNone, RegQWord, RegResourceList,
                               RegResourceRequirementsList, RegSZ)


class LoggerMock(object):
    def __init__(self):
        pass

    def info(self, msg):
        pass

    def warning(self, msg):
        pass

    def error(self, msg):
        pass


class TimestampMock(object):
    def __init__(self, t=1.5):
        self._t = t
        pass

    def timestamp(self):
        return self._t


class RegistryMock(object):
    def __init__(self, hive_name, hive_type, root):
        self._hive_name = hive_name
        self._hive_type = hive_type
        self._root = root

    def hive_name(self):
        return self._hive_name

    def hive_type(self):
        return self._hive_type

    def root(self):
        return self._root

    def open(self, path):
        return self.root().open(path)

    def set_ccs(self, num):
        select_key = RegistryKeyMock("Select", self.root())
        self.root().add_child(select_key)

        current_value = RegistryValueMock("Current", num, RegDWord)
        select_key.add_value(current_value)

    def dump(self):
        print("🌲")
        for subkey in self.root().subkeys():
            subkey.dump(1)


class RegistryKeyMock(object):
    def __init__(self, name, parent):
        self._name = name
        self._subkeys = []
        self._values = []
        self._parent = parent

    def build(path):
        before, _, last = path.rpartition("\\")
        if not before:
            # We're the root key
            root = RegistryKeyMock("ROOT", None)
            k = RegistryKeyMock(path, root)
            root.add_child(k)
            return k

        parent = RegistryKeyMock.build(before)
        k = RegistryKeyMock(last, parent)
        parent.add_child(k)
        return k

    def open(self, path):
        before, _, after = path.partition("\\")
        if not before:
            return self

        for subkey in self.subkeys():
            if subkey.name().lower() == before.lower():
                return subkey.open(after)

        raise RegistryKeyNotFoundException("Not found: " + path)

    def add_child(self, k):
        self._subkeys.append(k)

    def add_value(self, v):
        self._values.append(v)

    def subkeys(self):
        return self._subkeys

    def values(self):
        return self._values

    def name(self):
        return self._name

    def value(self, name):
        for v in self.values():
            if v.name().lower() == name.lower():
                return v
        raise RegistryValueNotFoundException("Could not find " + name)

    def subkey(self, name):
        for s in self.subkeys():
            if s.name().lower() == name.lower():
                return s

        raise RegistryKeyNotFoundException("Could not find " + name)

    def path(self):
        if self._parent:
            ppath = self._parent.path()
            return ppath + "\\" + self.name()
        else:
            return "\\" + self.name()

    def timestamp(self):
        return TimestampMock()

    def root(self):
        if self._parent:
            return self._parent.root()
        else:
            return self

    def dump(self, depth=0):
        print("\t" * depth + "📁", self.name())
        for v in self.values():
            v.dump(depth + 1)

        for s in self.subkeys():
            s.dump(depth + 1)


class RegistryValueMock(object):
    def __init__(self, name, data, type):
        self._name = name
        self._data = data
        self._type = type

    def name(self):
        if self._name:
            return self._name
        else:
            return "(default)"

    def value(self):
        return self._data

    def value_type(self):
        return self._type

    def value_type_str(self):
        t = self._type
        if t == RegSZ:
            return "RegSZ"
        elif t == RegExpandSZ:
            return "RegExpandSZ"
        elif t == RegBin:
            return "RegBin"
        elif t == RegDWord:
            return "RegDWord"
        elif t == RegMultiSZ:
            return "RegMultiSZ"
        elif t == RegQWord:
            return "RegQWord"
        elif t == RegNone:
            return "RegNone"
        elif t == RegBigEndian:
            return "RegBigEndian"
        elif t == RegLink:
            return "RegLink"
        elif t == RegResourceList:
            return "RegResourceList"
        elif t == RegFullResourceDescriptor:
            return "RegFullResourceDescriptor"
        elif t == RegResourceRequirementsList:
            return "RegResourceRequirementsList"

    def dump(self, depth=0):
        print("\t" * depth + "🗒️", self.name())



#!/usr/bin/env python3

import os
import sys

import pkg_resources

RST_TEMPLATE = """
Plugin test coverage
====================

Here is a recap of which plugins are properly tested, and which are not. Ideally, we should
strive for 100% coverage :-)

To make this dream a reality, please attach a test file to your pull requests! This way, the 
percentage at least won't go down.

**Current percentage: {percent}%**

.. csv-table:: 
   :header: "Plugin name","Test status"

{table}
"""

PLUGINS_DIR = os.path.join("regrippy", "plugins")
TESTS_DIR = "tests"

if __name__ == "__main__":
    all_plugins = [
        f for f in os.listdir(PLUGINS_DIR) if f.endswith(".py") and f != "__init__.py"
    ]
    all_tests = [
        f for f in os.listdir(TESTS_DIR) if f.startswith("test_") and f.endswith(".py")
    ]

    total = len(all_plugins)
    tested = 0

    if len(sys.argv) == 2 and sys.argv[1] == "--badge":
        for plugin in all_plugins:
            test_name = "test_" + plugin
            if os.path.isfile(os.path.join(TESTS_DIR, test_name)):
                tested += 1

        ratio = tested / total
        color = "red"
        if ratio > 0.3:
            color = "orange"
        if ratio > 0.7:
            color = "green"
        if ratio == 1.0:
            color = "brightgreen"

        url = f"https://img.shields.io/badge/tests-{tested}%2F{total}-{color}.svg"
        print(url)

    else:
        table_rows = []

        for plugin in all_plugins:
            test_name = "test_" + plugin
            row = '   "' + plugin[: -len(".py")] + '",'
            if os.path.isfile(os.path.join(TESTS_DIR, test_name)):
                row += "OK"
                tested += 1
            else:
                row += "MISSING"

            table_rows.append(row)

        print(
            RST_TEMPLATE.format(
                table="\n".join(table_rows), percent=int((tested / total) * 100)
            )
        )



import pytest

from regrippy.plugins.tasks import Plugin as plugin

from .reg_mock import (
    LoggerMock,
    RegBin,
    RegistryKeyMock,
    RegistryMock,
    RegistryValueMock,
    RegSZ,
)


def reg_hex_to_binary(reg_hex_str):
    if not reg_hex_str.startswith("hex:"):
        raise ValueError("hex string must start with 'hex:'")

    reg_hex_str = reg_hex_str[len("hex:") :]
    reg_hex_str = reg_hex_str.replace("\n", "").replace("\r", "").replace(" ", "")

    bytes = reg_hex_str.split(",")
    return bytearray([int(b, 16) for b in bytes])


@pytest.fixture
def mock_reg():
    k = RegistryKeyMock.build("Microsoft\\Windows NT\\CurrentVersion")
    r = RegistryMock("SOFTWARE", "SOFTWARE", k.root())

    # Set Windows version to Windows 7
    # TODO: create a mock registry from a Windows 10 sample (they have more information in the registry)
    k.add_value(RegistryValueMock("CurrentBuild", "7601", RegSZ))

    # Prepare the "TaskCache" key and its children
    schedule = RegistryKeyMock("Schedule", k)
    k.add_child(schedule)

    taskcache = RegistryKeyMock("TaskCache", schedule)
    schedule.add_child(taskcache)

    tasks = RegistryKeyMock("Tasks", taskcache)
    taskcache.add_child(tasks)

    plain = RegistryKeyMock("Plain", taskcache)
    taskcache.add_child(plain)

    # Create a couple scheduled tasks
    ## Mozilla Firefox's Check Default Browser
    guid = "{3DC1692F-C720-4D79-BC69-9032ABF0CB22}"
    path = "\\Mozilla\\Firefox Default Browser Agent 308046B0AF4A39CB"
    dynamic_info = reg_hex_to_binary(
        "hex:03,00,00,00,05,08,de,1b,01,f0,d7,01,9f,28,ba,bc,14,23,d8,01,\
  00,00,00,00,00,00,00,00"
    )
    triggers = reg_hex_to_binary(
        "hex:15,00,00,00,00,00,00,00,00,1b,97,01,00,00,00,00,00,6d,df,f7,00,\
  f0,d7,01,00,1b,97,01,00,00,00,00,ff,ff,ff,ff,ff,ff,ff,ff,48,21,41,00,48,48,\
  48,48,80,e3,dd,5d,48,48,48,48,00,48,48,48,48,48,48,48,00,48,48,48,48,48,48,\
  48,01,00,00,00,48,48,48,48,1c,00,00,00,48,48,48,48,01,05,00,00,00,00,00,05,\
  15,00,00,00,a7,33,50,76,4e,9e,21,69,fe,d5,fb,78,e9,03,00,00,48,48,48,48,1a,\
  00,00,00,48,48,48,48,4a,00,6f,00,68,00,6e,00,2d,00,50,00,43,00,5c,00,4a,00,\
  6f,00,68,00,6e,00,00,00,48,48,48,48,48,48,38,00,00,00,48,48,48,48,58,02,00,\
  00,10,0e,00,00,34,08,00,00,ff,ff,ff,ff,07,00,00,00,00,00,00,00,00,00,00,00,\
  00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,\
  00,00,00,dd,dd,00,00,00,00,00,00,00,1b,97,01,00,00,00,00,00,6d,df,f7,00,f0,\
  d7,01,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,\
  00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,ff,ff,ff,ff,01,00,00,00,\
  01,00,00,00,00,00,00,00,00,01,00,00,01,00,00,00,00,00,00,00,00,00,00,00"
    )
    hash = reg_hex_to_binary(
        "hex:67,2c,f8,7b,8e,f1,20,d1,7b,69,47,f3,9b,e3,c1,a6,53,5d,25,6d,71,3d,\
  38,97,2e,69,63,e9,85,61,74,15"
    )

    task_k = RegistryKeyMock(guid, tasks)
    tasks.add_child(task_k)
    task_k.add_value(RegistryValueMock("DynamicInfo", dynamic_info, RegBin))
    task_k.add_value(RegistryValueMock("Hash", hash, RegBin))
    task_k.add_value(RegistryValueMock("Path", path, RegSZ))
    task_k.add_value(RegistryValueMock("Triggers", triggers, RegBin))

    task_k = RegistryKeyMock(guid, plain)
    plain.add_child(task_k)

    return r


def test_tasks(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SOFTWARE", "-")

    results = list(p.run())

    assert len(results) == 1, "There should only be a single result"
    assert (
        results[0].custom["Path"]
        == "%System32%\\Tasks\\Mozilla\\Firefox Default Browser Agent 308046B0AF4A39CB"
    ), "The Task file path should be correctly parsed"
    assert results[0].custom["Actions"]



from regrippy import BasePlugin, PluginResult


class Plugin(BasePlugin):
    """Gets the temporary SRUM data from the registry"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        key = self.open_key("Microsoft\\Windows\\CurrentVersion\\SRUM\\Extensions\\")
        if not key:
            return

        for extension_key in key.subkeys():
            v = extension_key.value("(default)")
            res = PluginResult(key=extension_key, value=v)
            yield res



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Returns the computer name"""

    __REGHIVE__ = "SYSTEM"

    def run(self):
        key = self.open_key(
            self.get_currentcontrolset_path() + "\\Control\\ComputerName\\ComputerName"
        )
        if not key:
            return

        compname = key.value("ComputerName")

        res = PluginResult(key=key, value=compname)
        yield res

    def display_human(self, result):
        print(result.value_data)

    def display_machine(self, result):
        print(mactime(name=result.value_data, mtime=result.mtime))



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.timezone import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("ControlSet001\\Control\\TimeZoneInformation")
    reg = RegistryMock("SYSTEM", "system", key.root())
    reg.set_ccs(1)

    val = RegistryValueMock("TimeZoneKeyName", "SampleTimezone", RegSZ)
    key.add_value(val)

    return reg


def test_timezone(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SYSTEM", "-")

    results = list(p.run())

    assert len(results) == 1, "There should be a single result"
    assert (
        results[0].value_name == "TimeZoneKeyName"
    ), "It should have found the TimeZoneKeyName value"
    assert (
        results[0].value_data == "SampleTimezone"
    ), "It should have fetched the correct timezone value"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.runmru import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\RunMRU"
    )
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    mru_list = RegistryValueMock("MRUList", "bca", RegSZ)
    key.add_value(mru_list)

    a = RegistryValueMock("a", "third result", RegSZ)
    b = RegistryValueMock("b", "first result", RegSZ)
    c = RegistryValueMock("c", "second result", RegSZ)

    key.add_value(a)
    key.add_value(b)
    key.add_value(c)

    return reg


def test_runmru(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == 3, "There should be 3 results"

    assert results[0].value_name == "b", "The first result key should be 'b'"
    assert (
        results[0].value_data == "first result"
    ), "The first result should be 'first result'"

    assert results[1].value_name == "c", "The second result key should be 'c'"
    assert (
        results[1].value_data == "second result"
    ), "The second result should be 'second result'"

    assert results[2].value_name == "a", "The third result key should be 'a'"
    assert (
        results[2].value_data == "third result"
    ), "The third result should be 'third result'"



import pytest
from Registry.Registry import RegBin

from regrippy.plugins.userassist import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg_seven():
    key = RegistryKeyMock.build(
        "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\UserAssist"
    )
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    guid = RegistryKeyMock("{CEBFF5CD-ACE2-4F4F-9178-9926F41749EA}", key)
    key.add_child(guid)

    count = RegistryKeyMock("Count", guid)
    guid.add_child(count)

    notepad = RegistryValueMock(
        "{1NP14R77-02R7-4R5Q-O744-2RO1NR5198O7}\\abgrcnq.rkr",
        bytearray(
            [
                0x00,
                0x00,
                0x00,
                0x00,
                0x06,
                0x00,
                0x00,
                0x00,
                0x05,
                0x00,
                0x00,
                0x00,
                0x60,
                0xEA,
                0x00,
                0x00,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0x00,
                0x00,
                0x80,
                0xBF,
                0xFF,
                0xFF,
                0xFF,
                0xFF,
                0x02,
                0xD2,
                0x7E,
                0xCE,
                0xB4,
                0xBF,
                0xD4,
                0x01,
                0x00,
                0x00,
                0x00,
                00,
            ]
        ),
        RegBin,
    )
    count.add_value(notepad)

    return reg


@pytest.fixture
def mock_reg_xp():
    key = RegistryKeyMock.build(
        "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\UserAssist"
    )
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    guid = RegistryKeyMock("{75048700-EF1F-11D0-9888-006097DEACF9}", key)
    key.add_child(guid)

    count = RegistryKeyMock("Count", guid)
    guid.add_child(count)

    notepad = RegistryValueMock(
        "HRZR_EHACNGU:P:\\JVAQBJF\\flfgrz32\\abgrcnq.rkr",
        bytearray(
            [
                0x01,
                0x00,
                0x00,
                0x00,
                0x08,
                0x00,
                0x00,
                0x00,
                0xC0,
                0x66,
                0x16,
                0xCE,
                0xA4,
                0xCE,
                0xD4,
                0x01,
            ]
        ),
        RegBin,
    )
    count.add_value(notepad)

    return reg


def test_userassist_win7(mock_reg_seven):
    p = plugin(mock_reg_seven, LoggerMock(), "NTUSER.DAT", "-")
    results = list(p.run())

    assert len(results) == 1, "There should be a single result"
    assert (
        "userassist" in results[0].custom.keys()
    ), "A 'userassist' object should have been created"

    ua = results[0].custom["userassist"]

    assert ua.name.endswith("\\notepad.exe"), "Decoded name should be notepad"
    assert ua.number_of_execs == 6, "Notepad was executed 6 times"
    assert ua.focus_count == 5, "Notepad was focused 5 times"
    assert ua.focus_time_secs == 0xEA60, "Notepad was focused for 60,000 seconds (!!)"


def test_userassist_winxp(mock_reg_xp):
    p = plugin(mock_reg_xp, LoggerMock(), "NTUSER.DAT", "-")
    results = list(p.run())

    assert len(results) == 1, "There should be a single result"
    assert (
        "userassist" in results[0].custom.keys()
    ), "A 'userassist' object should have been created"

    ua = results[0].custom["userassist"]

    assert ua.name.endswith("\\notepad.exe"), "Decoded name should be notepad"
    assert ua.number_of_execs == 3, "Notepad was executed 3 times"



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Lists 'My Recent Documents'"""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        key = self.open_key(
            r"Software\Microsoft\Windows\CurrentVersion\Explorer\RecentDocs"
        )
        if not key:
            return

        for v in key.values():
            if v.name() == "MRUListEx":
                continue

            binary = v.value()
            offset_str_end = binary.find(b"\x00\x00")
            docname = binary[: offset_str_end + 1].decode("utf16")

            res = PluginResult(key=key, value=v)
            res.custom["docname"] = docname
            yield res

    def display_human(self, result):
        print(result.custom["docname"])

    def display_machine(self, result):
        print(
            mactime(
                name=f"{self.guess_username()}\t{result.custom['docname']}",
                mtime=result.mtime,
            )
        )



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.typedurls import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("Software\\Microsoft\\Internet Explorer\\TypedURLs")
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    url1 = RegistryValueMock("url1", "https://outlook.com", RegSZ)
    url2 = RegistryValueMock("url2", "https://airbus.com/order", RegSZ)

    key.add_value(url1)
    key.add_value(url2)

    return reg


def test_typedurls(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == 2
    assert results[0].value_data == "https://outlook.com", "First URL should be Outlook"
    assert (
        results[1].value_data == "https://airbus.com/order"
    ), "Second URL should be Airbus"



import pytest
from Registry.Registry import RegDWord, RegSZ

from regrippy.plugins.proxy import Plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        r"Software\Microsoft\Windows\CurrentVersion\Internet Settings"
    )
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    # Add a normal proxy config
    url = RegistryValueMock("ProxyServer", "172.16.2.34:8080", RegSZ)
    enabled = RegistryValueMock("ProxyEnabled", 1, RegDWord)
    exceptions = RegistryValueMock("ProxyOverride", "10.0.0.0/8", RegSZ)

    key.add_value(url)
    key.add_value(enabled)
    key.add_value(exceptions)

    # Add a Proxy.pac
    pac = RegistryValueMock(
        "AutoConfigURL", "http://proxy.internal.corp/proxy.pac", RegSZ
    )

    key.add_value(pac)

    return reg


def test_proxy(mock_reg):
    p = Plugin(mock_reg, LoggerMock, "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == 2, "There should be 2 results: a simple proxy and a proxypac"

    proxy = [x for x in results if x.custom["type"] == "proxy"][0]
    pac = [x for x in results if x.custom["type"] == "autoconfig"][0]

    assert proxy.custom["enabled"], "Proxy should be enabled"
    assert (
        proxy.custom["proxy"] == "172.16.2.34:8080"
    ), "Proxy should be set to '172.16.2.34:8080'"
    assert (
        proxy.custom["exceptions"] == "10.0.0.0/8"
    ), "There should be a single exception"

    assert (
        pac.custom["proxypac"] == "http://proxy.internal.corp/proxy.pac"
    ), "The ProxyPAC should be properly configured"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.usersids import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        "Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\S-1-5-18"
    )
    reg = RegistryMock("SOFTWARE", "software", key.root())

    val = RegistryValueMock("ProfileImagePath", "systemprofile", RegSZ)
    key.add_value(val)

    return reg


def test_usersids(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SOFTWARE", "-")

    results = list(p.run())
    assert len(results) == 1, "There should be a single result"
    assert (
        results[0].custom["value"] == "systemprofile:	S-1-5-18"
    ), "systemprofile:	S-1-5-18"



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Retrieves all Scheduled Tasks in the registry cache"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        k = self.open_key(r"Microsoft\Windows NT\CurrentVersion\Schedule\TaskCache")
        if not k:
            return

        version_k = self.open_key(r"Microsoft\Windows NT\CurrentVersion")
        if not version_k:
            return

        current_build = int(version_k.value("CurrentBuild").value())
        task_prefix = "%WINDIR%"
        if current_build >= 6002:
            # Since Windows Vista (Task Scheduler 2.0), tasks are stored in %System32%
            task_prefix = "%System32%"

        boot_guids = []
        logon_guids = []
        maintenance_guids = []
        plain_guids = []

        for t in ["Boot", "Logon", "Maintenance", "Plain"]:
            if t not in [x.name() for x in k.subkeys()]:
                continue
            for guid in k.subkey(t).subkeys():
                g = guid.name()
                if t == "Boot":
                    boot_guids.append(g)
                elif t == "Logon":
                    logon_guids.append(g)
                elif t == "Maintenance":
                    maintenance_guids.append(g)
                elif t == "Plain":
                    plain_guids.append(g)

        for task in k.subkey("Tasks").subkeys():
            guid = task.name()
            runtype = "N/A"
            if guid in boot_guids:
                runtype = "Boot"
            elif guid in logon_guids:
                runtype = "Logon"
            elif guid in maintenance_guids:
                runtype = "Maintenance"
            elif guid in plain_guids:
                runtype = "Plain"

            r = PluginResult(key=task)
            task_values = [x.name() for x in task.values()]
            r.custom = {
                "RunType": runtype,
                "Path": f"{task_prefix}\\Tasks{task.value('Path').value()}",
                "Actions": RegistryAction.from_binary(task.value("Actions").value())
                if "Actions" in task_values
                else RegistryAction(None, None),
                "Source": task.value("Source").value()
                if "Source" in task_values
                else None,
                "Author": task.value("Author").value()
                if "Author" in task_values
                else None,
                "Description": task.value("Description").value()
                if "Description" in task_values
                else None,
            }
            yield r

    def display_human(self, r):
        print(r.custom["Path"])
        if r.custom["Author"]:
            print(f"\tAuthor: {r.custom['Author']}")
        if r.custom["Actions"].runas:
            print(f"\tRuns as: {r.custom['Actions'].runas}")
        if r.custom["Source"]:
            print(f"\tSource: {r.custom['Source']}")
        if r.custom["Actions"].cmd:
            print(f"\tAction: {r.custom['Actions'].cmd}")
        if r.custom["Description"]:
            print(f"\tDescription: {r.custom['Description']}")
        print()


class RegistryAction(object):
    def __init__(self, runas, cmd):
        self.runas = runas
        self.cmd = cmd

    def __str__(self):
        return f"RegistryAction<runas={self.runas}, cmd={self.cmd}>"

    @staticmethod
    def from_binary(binary):
        name_len = int.from_bytes(binary[2:6], byteorder="little")
        name = binary[6 : 6 + name_len].decode("utf-16-le")

        offset = 6 + name_len
        action_type = binary[offset : offset + 2].decode("ascii")
        cmd = None
        offset += 2

        if action_type == "ff":
            offset += 4
            cmd_len = int.from_bytes(binary[offset : offset + 4], byteorder="little")
            offset += 4
            cmd = binary[offset : offset + cmd_len].decode("utf-16-le")

            offset += cmd_len
            args_len = int.from_bytes(binary[offset : offset + 4], byteorder="little")
            if args_len > 0:
                offset += 4
                args = binary[offset : offset + args_len].decode("utf-16-le")
                cmd += " " + args

        return RegistryAction(name, cmd)



from Registry.Registry import RegistryValueNotFoundException

from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """list all GPOs applied on this system"""

    __REGHIVE__ = ["SOFTWARE", "NTUSER.DAT"]

    EXTENSIONS = {
        "{35378EAC-683F-11D2-A89A-00C04FBBCFA2}": "Registry Settings",
    }

    def run(self):
        path = r"Microsoft\Windows\CurrentVersion\Group Policy\History"
        if self.hive_name == "NTUSER.DAT":
            path = "SOFTWARE\\" + path
        else:
            self.get_gp_extensions()

        k = self.open_key(path)
        if not k:
            return

        for subkey in k.subkeys():
            for gpo in subkey.subkeys():
                r = PluginResult(key=gpo)
                r.custom = {
                    "Extension": self.EXTENSIONS.get(subkey.name(), subkey.name()),
                    "DisplayName": gpo.value("DisplayName").value(),
                    "DSPath": (
                        gpo.value("DSPath").value()
                        if "DSPath" in [x.name() for x in gpo.values()]
                        else "N/A"
                    ),
                    "Path": gpo.value("FileSysPath").value(),
                    "GPOName": gpo.value("GPOName").value(),
                    "IParam": (
                        gpo.value("IParam").value()
                        if "IParam" in [x.name() for x in gpo.values()]
                        else "N/A"
                    ),
                    "Options": (
                        gpo.value("Options").value()
                        if "Options" in [x.name() for x in gpo.values()]
                        else "N/A"
                    ),
                }
                yield r

    def get_gp_extensions(self):
        k = self.open_key(r"Microsoft\Windows NT\CurrentVersion\Winlogon\GPExtensions")
        if not k:
            return

        for subkey in k.subkeys():
            guid = subkey.name()
            if guid in self.EXTENSIONS.keys():
                continue

            name = guid
            try:
                name = subkey.value("(default)").value()
            except RegistryValueNotFoundException:
                pass
            self.EXTENSIONS[guid] = name

    def display_human(self, r):
        print(f"[{r.custom['Extension']}] {r.custom['DisplayName']}")
        print(f"\tPath: {r.custom['Path']}")
        print(f"\tDSPath: {r.custom['DSPath']}")
        print(f"\tGPOName: {r.custom['GPOName']}")
        print(f"\tIParam: {r.custom['IParam']}")
        print(f"\tOptions: {r.custom['Options']}")
        print()

    def display_machine(self, r):
        print(
            mactime(
                name=f"GPO: {r.custom['DisplayName']} [{r.custom['Extension']}] :: {r.custom['GPOName']}",
                mtime=r.mtime,
            )
        )



#!/usr/bin/env python
# ShimCacheParser.py
#
# Andrew Davis, andrew.davis@mandiant.com
# Copyright 2012 Mandiant
#
# Mandiant licenses this file to you under the Apache License, Version
# 2.0 (the "License"); you may not use this file except in compliance with the
# License.  You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.  See the License for the specific language governing
# permissions and limitations under the License.
#
# Identifies and parses Application Compatibility Shim Cache entries for forensic data.

import struct
import datetime
import codecs
from io import BytesIO
from csv import writer
import logging

logging.basicConfig()
logger = logging.getLogger("shimcacheparser")
logger.setLevel("ERROR")

# Values used by Windows 5.2 and 6.0 (Server 2003 through Vista/Server 2008)
CACHE_MAGIC_NT5_2 = 0xBADC0FFE
CACHE_HEADER_SIZE_NT5_2 = 0x8
NT5_2_ENTRY_SIZE32 = 0x18
NT5_2_ENTRY_SIZE64 = 0x20

# Values used by Windows 6.1 (Win7 and Server 2008 R2)
CACHE_MAGIC_NT6_1 = 0xBADC0FEE
CACHE_HEADER_SIZE_NT6_1 = 0x80
NT6_1_ENTRY_SIZE32 = 0x20
NT6_1_ENTRY_SIZE64 = 0x30
CSRSS_FLAG = 0x2

# Values used by Windows 5.1 (WinXP 32-bit)
WINXP_MAGIC32 = 0xDEADBEEF
WINXP_HEADER_SIZE32 = 0x190
WINXP_ENTRY_SIZE32 = 0x228
MAX_PATH = 520

# Values used by Windows 8
WIN8_STATS_SIZE = 0x80
WIN8_MAGIC = b"00ts"

# Magic value used by Windows 8.1
WIN81_MAGIC = b"10ts"

# Values used by Windows 10
WIN10_STATS_SIZE = 0x30
WIN10_CREATORS_STATS_SIZE = 0x34
WIN10_MAGIC = b"10ts"
CACHE_HEADER_SIZE_NT6_4 = 0x30
CACHE_MAGIC_NT6_4 = 0x30

bad_entry_data = "N/A"
g_verbose = False
g_usebom = False
output_header = ["Last Modified", "Last Update", "Path", "File Size", "Exec Flag"]

# Date Formats
DATE_MDY = "%m/%d/%y %H:%M:%S"
DATE_ISO = "%Y-%m-%d %H:%M:%S"
g_timeformat = DATE_ISO

# Shim Cache format used by Windows 5.2 and 6.0 (Server 2003 through Vista/Server 2008)
class CacheEntryNt5(object):
    def __init__(self, is32bit, data=None):

        self.is32bit = is32bit
        if data != None:
            self.update(data)

    def update(self, data):

        if self.is32bit:
            entry = struct.unpack("<2H 3L 2L", data)
        else:
            entry = struct.unpack("<2H 4x Q 2L 2L", data)
        self.wLength = entry[0]
        self.wMaximumLength = entry[1]
        self.Offset = entry[2]
        self.dwLowDateTime = entry[3]
        self.dwHighDateTime = entry[4]
        self.dwFileSizeLow = entry[5]
        self.dwFileSizeHigh = entry[6]

    def size(self):

        if self.is32bit:
            return NT5_2_ENTRY_SIZE32
        else:
            return NT5_2_ENTRY_SIZE64


# Shim Cache format used by Windows 6.1 (Win7 through Server 2008 R2)
class CacheEntryNt6(object):
    def __init__(self, is32bit, data=None):

        self.is32bit = is32bit
        if data != None:
            self.update(data)

    def update(self, data):

        if self.is32bit:
            entry = struct.unpack("<2H 7L", data)
        else:
            entry = struct.unpack("<2H 4x Q 4L 2Q", data)
        self.wLength = entry[0]
        self.wMaximumLength = entry[1]
        self.Offset = entry[2]
        self.dwLowDateTime = entry[3]
        self.dwHighDateTime = entry[4]
        self.FileFlags = entry[5]
        self.Flags = entry[6]
        self.BlobSize = entry[7]
        self.BlobOffset = entry[8]

    def size(self):

        if self.is32bit:
            return NT6_1_ENTRY_SIZE32
        else:
            return NT6_1_ENTRY_SIZE64


# Convert FILETIME to datetime.
# Based on http://code.activestate.com/recipes/511425-filetime-to-datetime/
def convert_filetime(dwLowDateTime, dwHighDateTime):

    try:
        date = datetime.datetime(1601, 1, 1, 0, 0, 0)
        temp_time = dwHighDateTime
        temp_time <<= 32
        temp_time |= dwLowDateTime
        return date + datetime.timedelta(microseconds=temp_time / 10)
    except OverflowError:
        return None


# Return a unique list while preserving ordering.
def unique_list(li):

    ret_list = []
    for entry in li:
        if entry not in ret_list:
            ret_list.append(entry)
    return ret_list


# Write the Log.
def write_it(rows, outfile=None):

    try:

        if not rows:
            logger.error("[-] No data to write...")
            return

        if not outfile:
            for row in rows:
                print(" ".join(["%s" % x for x in row]))
        else:
            logger.debug("[+] Writing output to %s..." % outfile)
            try:
                f = open(outfile, "wb")
                if g_usebom:
                    f.write(codecs.BOM_UTF8)
                csv_writer = writer(f, delimiter=",")
                csv_writer.writerows(rows)
                f.close()
            except IOError as err:
                logger.error("[-] Error writing output file: %s" % str(err))
                return

    except UnicodeEncodeError as err:
        logger.error("[-] Error writing output file: %s" % str(err))
        return


# Read the Shim Cache format, return a list of last modified dates/paths.
def read_cache(cachebin):

    if len(cachebin) < 16:
        # Data size less than minimum header size.
        return None

    try:
        # Get the format type
        magic = struct.unpack("<L", cachebin[0:4])[0]

        # This is a Windows 2k3/Vista/2k8 Shim Cache format,
        if magic == CACHE_MAGIC_NT5_2:

            # Shim Cache types can come in 32-bit or 64-bit formats. We can
            # determine this because 64-bit entries are serialized with u_int64
            # pointers. This means that in a 64-bit entry, valid UNICODE_STRING
            # sizes are followed by a NULL DWORD. Check for this here.
            test_size = struct.unpack("<H", cachebin[8:10])[0]
            test_max_size = struct.unpack("<H", cachebin[10:12])[0]
            if (
                test_max_size - test_size == 2
                and struct.unpack("<L", cachebin[12:16])[0]
            ) == 0:
                logger.debug("[+] Found 64bit Windows 2k3/Vista/2k8 Shim Cache data...")
                entry = CacheEntryNt5(False)
                return read_nt5_entries(cachebin, entry)

            # Otherwise it's 32-bit data.
            else:
                logger.debug("[+] Found 32bit Windows 2k3/Vista/2k8 Shim Cache data...")
                entry = CacheEntryNt5(True)
                return read_nt5_entries(cachebin, entry)

        # This is a Windows 7/2k8-R2 Shim Cache.
        elif magic == CACHE_MAGIC_NT6_1:
            test_size = struct.unpack(
                "<H", cachebin[CACHE_HEADER_SIZE_NT6_1 : CACHE_HEADER_SIZE_NT6_1 + 2]
            )[0]
            test_max_size = struct.unpack(
                "<H",
                cachebin[CACHE_HEADER_SIZE_NT6_1 + 2 : CACHE_HEADER_SIZE_NT6_1 + 4],
            )[0]

            # Shim Cache types can come in 32-bit or 64-bit formats.
            # We can determine this because 64-bit entries are serialized with
            # u_int64 pointers. This means that in a 64-bit entry, valid
            # UNICODE_STRING sizes are followed by a NULL DWORD. Check for this here.
            if (
                test_max_size - test_size == 2
                and struct.unpack(
                    "<L",
                    cachebin[CACHE_HEADER_SIZE_NT6_1 + 4 : CACHE_HEADER_SIZE_NT6_1 + 8],
                )[0]
            ) == 0:
                logger.debug("[+] Found 64bit Windows 7/2k8-R2 Shim Cache data...")
                entry = CacheEntryNt6(False)
                return read_nt6_entries(cachebin, entry)
            else:
                logger.debug("[+] Found 32bit Windows 7/2k8-R2 Shim Cache data...")
                entry = CacheEntryNt6(True)
                return read_nt6_entries(cachebin, entry)

        # This is WinXP cache data
        elif magic == WINXP_MAGIC32:
            logger.debug("[+] Found 32bit Windows XP Shim Cache data...")
            return read_winxp_entries(cachebin)

        # Check the data set to see if it matches the Windows 8 format.
        elif (
            len(cachebin) > WIN8_STATS_SIZE
            and cachebin[WIN8_STATS_SIZE : WIN8_STATS_SIZE + 4] == WIN8_MAGIC
        ):
            logger.debug("[+] Found Windows 8/2k12 Apphelp Cache data...")
            return read_win8_entries(cachebin, WIN8_MAGIC)

        # Windows 8.1 will use a different magic dword, check for it
        elif (
            len(cachebin) > WIN8_STATS_SIZE
            and cachebin[WIN8_STATS_SIZE : WIN8_STATS_SIZE + 4] == WIN81_MAGIC
        ):
            logger.debug("[+] Found Windows 8.1 Apphelp Cache data...")
            return read_win8_entries(cachebin, WIN81_MAGIC)

        # Windows 10 will use a different magic dword, check for it
        elif (
            len(cachebin) > WIN10_STATS_SIZE
            and cachebin[WIN10_STATS_SIZE : WIN10_STATS_SIZE + 4] == WIN10_MAGIC
        ):
            logger.debug("[+] Found Windows 10 Apphelp Cache data...")
            return read_win10_entries(cachebin, WIN10_MAGIC)

        # Windows 10 Creators Update will use a different STATS_SIZE, account for it
        elif (
            len(cachebin) > WIN10_CREATORS_STATS_SIZE
            and cachebin[WIN10_CREATORS_STATS_SIZE : WIN10_CREATORS_STATS_SIZE + 4]
            == WIN10_MAGIC
        ):
            logger.debug("[+] Found Windows 10 Creators Update Apphelp Cache data...")
            return read_win10_entries(cachebin, WIN10_MAGIC, creators_update=True)

        else:
            logger.error(
                "[-] Got an unrecognized magic value of 0x%x... bailing" % magic
            )
            return None

    except (RuntimeError, TypeError, NameError) as err:
        logger.error("[-] Error reading Shim Cache data: %s" % err)
        return None


# Read Windows 8/2k12/8.1 Apphelp Cache entry formats.
def read_win8_entries(bin_data, ver_magic):
    offset = 0
    entry_meta_len = 12
    entry_list = []

    # Skip past the stats in the header
    cache_data = bin_data[WIN8_STATS_SIZE:]

    data = BytesIO(cache_data)
    while data.tell() < len(cache_data):
        header = data.read(entry_meta_len)
        # Read in the entry metadata
        # Note: the crc32 hash is of the cache entry data
        magic, crc32_hash, entry_len = struct.unpack("<4sLL", header)

        # Check the magic tag
        if magic != ver_magic:
            raise Exception(
                "Invalid version magic tag found: 0x%x" % struct.unpack("<L", magic)[0]
            )

        entry_data = BytesIO(data.read(entry_len))

        # Read the path length
        path_len = struct.unpack("<H", entry_data.read(2))[0]
        if path_len == 0:
            path = "None"
        else:
            path = (
                entry_data.read(path_len).decode("utf-16le", "replace").encode("utf-8")
            )

        # Check for package data
        package_len = struct.unpack("<H", entry_data.read(2))[0]
        if package_len > 0:
            # Just skip past the package data if present (for now)
            entry_data.seek(package_len, 1)

        # Read the remaining entry data
        flags, unk_1, low_datetime, high_datetime, unk_2 = struct.unpack(
            "<LLLLL", entry_data.read(20)
        )

        # Check the flag set in CSRSS
        if flags & CSRSS_FLAG:
            exec_flag = "True"
        else:
            exec_flag = "False"

        last_mod_date = convert_filetime(low_datetime, high_datetime)
        try:
            last_mod_date = last_mod_date.strftime(g_timeformat)
        except ValueError:
            last_mod_date = bad_entry_data

        row = [last_mod_date, "N/A", path, "N/A", exec_flag]
        entry_list.append(row)

    return entry_list


# Read Windows 10 Apphelp Cache entry format
def read_win10_entries(bin_data, ver_magic, creators_update=False):

    offset = 0
    entry_meta_len = 12
    entry_list = []

    # Skip past the stats in the header
    if creators_update:
        cache_data = bin_data[WIN10_CREATORS_STATS_SIZE:]
    else:
        cache_data = bin_data[WIN10_STATS_SIZE:]

    data = BytesIO(cache_data)
    while data.tell() < len(cache_data):
        header = data.read(entry_meta_len)
        # Read in the entry metadata
        # Note: the crc32 hash is of the cache entry data
        magic, crc32_hash, entry_len = struct.unpack("<4sLL", header)

        # Check the magic tag
        if magic != ver_magic:
            raise Exception(
                "Invalid version magic tag found: 0x%x" % struct.unpack("<L", magic)[0]
            )

        entry_data = BytesIO(data.read(entry_len))

        # Read the path length
        path_len = struct.unpack("<H", entry_data.read(2))[0]
        if path_len == 0:
            path = "None"
        else:
            path = entry_data.read(path_len).decode("utf-16le", "replace")

        # Read the remaining entry data
        low_datetime, high_datetime = struct.unpack("<LL", entry_data.read(8))

        last_mod_date = convert_filetime(low_datetime, high_datetime)
        try:
            last_mod_date = last_mod_date.strftime(g_timeformat)
        except ValueError:
            last_mod_date = bad_entry_data

        # Skip the unrecognized Microsoft App entry format for now
        if last_mod_date == bad_entry_data:
            continue

        row = [last_mod_date, "N/A", path, "N/A", "N/A"]
        entry_list.append(row)

    return entry_list


# Read Windows 2k3/Vista/2k8 Shim Cache entry formats.
def read_nt5_entries(bin_data, entry):

    try:
        entry_list = []
        contains_file_size = False
        entry_size = entry.size()
        exec_flag = ""

        num_entries = struct.unpack("<L", bin_data[4:8])[0]
        if num_entries == 0:
            return None

        # On Windows Server 2008/Vista, the filesize is swapped out of this
        # structure with two 4-byte flags. Check to see if any of the values in
        # "dwFileSizeLow" are larger than 2-bits. This indicates the entry contained file sizes.
        for offset in range(
            CACHE_HEADER_SIZE_NT5_2,
            (num_entries * entry_size) + CACHE_HEADER_SIZE_NT5_2,
            entry_size,
        ):

            entry.update(bin_data[offset : offset + entry_size])

            if entry.dwFileSizeLow > 3:
                contains_file_size = True
                break

        # Now grab all the data in the value.
        for offset in range(
            CACHE_HEADER_SIZE_NT5_2,
            (num_entries * entry_size) + CACHE_HEADER_SIZE_NT5_2,
            entry_size,
        ):

            entry.update(bin_data[offset : offset + entry_size])

            last_mod_date = convert_filetime(entry.dwLowDateTime, entry.dwHighDateTime)
            try:
                last_mod_date = last_mod_date.strftime(g_timeformat)
            except ValueError:
                last_mod_date = bad_entry_data
            path = bin_data[entry.Offset : entry.Offset + entry.wLength].decode(
                "utf-16le", "replace"
            )

            # It contains file size data.
            if contains_file_size:
                hit = [last_mod_date, "N/A", path, str(entry.dwFileSizeLow), "N/A"]
                if hit not in entry_list:
                    entry_list.append(hit)

            # It contains flags.
            else:
                # Check the flag set in CSRSS
                if entry.dwFileSizeLow & CSRSS_FLAG:
                    exec_flag = "True"
                else:
                    exec_flag = "False"

                hit = [last_mod_date, "N/A", path, "N/A", exec_flag]
                if hit not in entry_list:
                    entry_list.append(hit)

        return entry_list

    except (RuntimeError, ValueError, NameError) as err:
        logger.error("[-] Error reading Shim Cache data: %s..." % err)
        return None


# Read the Shim Cache Windows 7/2k8-R2 entry format,
# return a list of last modifed dates/paths.
def read_nt6_entries(bin_data, entry):

    try:
        entry_list = []
        exec_flag = ""
        entry_size = entry.size()
        num_entries = struct.unpack("<L", bin_data[4:8])[0]

        if num_entries == 0:
            return None

        # Walk each entry in the data structure.
        for offset in range(
            CACHE_HEADER_SIZE_NT6_1,
            num_entries * entry_size + CACHE_HEADER_SIZE_NT6_1,
            entry_size,
        ):

            entry.update(bin_data[offset : offset + entry_size])
            last_mod_date = convert_filetime(entry.dwLowDateTime, entry.dwHighDateTime)
            try:
                last_mod_date = last_mod_date.strftime(g_timeformat)
            except ValueError:
                last_mod_date = "N/A"
            path = bin_data[entry.Offset : entry.Offset + entry.wLength].decode(
                "utf-16le", "replace"
            )

            # Test to see if the file may have been executed.
            if entry.FileFlags & CSRSS_FLAG:
                exec_flag = "True"
            else:
                exec_flag = "False"

            hit = [last_mod_date, "N/A", path, "N/A", exec_flag]

            if hit not in entry_list:
                entry_list.append(hit)
        return entry_list

    except (RuntimeError, ValueError, NameError) as err:
        logger.error("[-] Error reading Shim Cache data: %s..." % err)
        return None


# Read the WinXP Shim Cache data. Some entries can be missing data but still
# contain useful information, so try to get as much as we can.
def read_winxp_entries(bin_data):

    entry_list = []

    try:

        num_entries = struct.unpack("<L", bin_data[8:12])[0]
        if num_entries == 0:
            return None

        for offset in range(
            WINXP_HEADER_SIZE32,
            (num_entries * WINXP_ENTRY_SIZE32) + WINXP_HEADER_SIZE32,
            WINXP_ENTRY_SIZE32,
        ):

            # No size values are included in these entries, so search for utf-16 terminator.
            path_len = bin_data[offset : offset + (MAX_PATH + 8)].find("\x00\x00")

            # if path is corrupt, procede to next entry.
            if path_len == 0:
                continue
            path = bin_data[offset : offset + path_len + 1].decode("utf-16le")

            entry_data = offset + (MAX_PATH + 8)

            # Get last mod time.
            last_mod_time = struct.unpack("<2L", bin_data[entry_data : entry_data + 8])
            try:
                last_mod_time = convert_filetime(
                    last_mod_time[0], last_mod_time[1]
                ).strftime(g_timeformat)
            except ValueError:
                last_mod_time = "N/A"

            # Get last file size.
            file_size = struct.unpack(
                "<2L", bin_data[entry_data + 8 : entry_data + 16]
            )[0]
            if file_size == 0:
                file_size = bad_entry_data

            # Get last update time.
            exec_time = struct.unpack(
                "<2L", bin_data[entry_data + 16 : entry_data + 24]
            )
            try:
                exec_time = convert_filetime(exec_time[0], exec_time[1]).strftime(
                    g_timeformat
                )
            except ValueError:
                exec_time = bad_entry_data

            hit = [last_mod_time, exec_time, path, file_size, "N/A"]
            if hit not in entry_list:
                entry_list.append(hit)
        return entry_list

    except (RuntimeError, ValueError, NameError) as err:
        logger.error("[-] Error reading Shim Cache data %s" % err)
        return None



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.compname import Plugin as plugin

from .reg_mock import LoggerMock, RegistryKeyMock, RegistryMock, RegistryValueMock


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("ControlSet001\\Control\\ComputerName\\ComputerName")
    reg = RegistryMock("SYSTEM", "system", key.root())
    reg.set_ccs(1)

    val = RegistryValueMock("ComputerName", "MyCoolPC", RegSZ)
    key.add_value(val)

    return reg


def test_compname(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SYSTEM", "-")

    results = list(p.run())

    assert len(results) == 1, "There should be a single result"
    assert (
        results[0].value_data == "MyCoolPC"
    ), "The extracted computer name should match 'MyCoolPC'"



from regrippy import BasePlugin, mactime, PluginResult
from Registry.Registry import (
    RegistryKeyNotFoundException,
    RegistryValueNotFoundException,
)


class Plugin(BasePlugin):
    """Lists information about PuTTY connections: saved public keys, saved sessions, etc."""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        k = self.open_key(r"Software\SimonTatham\PuTTY")
        if not k:
            return

        # SSH host keys
        try:
            ssh_host_keys = k.subkey("SshHostKeys")
            for v in ssh_host_keys.values():
                name = v.name()
                crypto, host = name.split("@")

                r = PluginResult(key=ssh_host_keys, value=v)
                r.custom["type"] = "sshkey"
                r.custom["crypto"] = crypto
                r.custom["host"] = host

                yield r
        except RegistryKeyNotFoundException:
            self.warning("Could not find SshHostKeys subkey")

        # Saved sessions
        try:
            sessions = k.subkey("Sessions")
            for sess in sessions.subkeys():
                try:
                    name = sess.name()
                    host = sess.value("HostName").value()
                    proto = sess.value("Protocol").value()

                    r = PluginResult(key=sess)
                    r.custom["type"] = "session"
                    r.custom["host"] = host
                    r.custom["protocol"] = proto

                    yield r
                except RegistryValueNotFoundException:
                    self.warning(f"Malformed entry: {sess.name()}")
        except RegistryKeyNotFoundException:
            self.warning("Could not find Sessions subkey")

    def display_human(self, res):
        if res.custom["type"] == "sshkey":
            print(f"[SSH KEY] {res.custom['host']}\t(key type: {res.custom['crypto']})")
        elif res.custom["type"] == "session":
            print(
                f"[SESSION] \"{res.key_name}\" => {res.custom['host']}\t(protocol: {res.custom['protocol']})"
            )
        else:
            self.error(f"Unknown result type: {res.custom['type']}")

    def display_machine(self, res):
        if res.custom["type"] == "sshkey":
            print(
                mactime(
                    name=f"PuTTY SSH Key: {res.custom['host']} with {res.custom['crypto']}",
                    mtime=res.mtime,
                )
            )
        elif res.custom["type"] == "session":
            print(
                mactime(
                    name=f"PuTTY Session: {res.custom['host']} with {res.custom['protocol']}",
                    mtime=res.mtime,
                )
            )



from enum import Enum

from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Lists all services installed on the system"""

    __REGHIVE__ = "SYSTEM"

    def run(self):
        key = self.open_key(self.get_currentcontrolset_path() + "\\Services")
        if not key:
            return

        for service in key.subkeys():
            values = [x.name() for x in service.values()]

            res = PluginResult(key=service, value=None)
            res.custom = {
                "image_path": service.value("ImagePath").value()
                if "ImagePath" in values
                else "N/A",
                "start_mode": ServiceStartMode(
                    service.value("Start").value() if "Start" in values else -1
                ),
                "description": service.value("Description").value()
                if "Description" in values
                else "N/A",
            }
            yield res

    def display_human(self, result):
        print(result.key_name)
        print(f"\tDescription: {result.custom['description']}")
        print(f"\tImagePath: {result.custom['image_path']}")
        print(f"\tStart: {result.custom['start_mode']}")
        print()

    def display_machine(self, result):
        print(
            mactime(
                name=f"{result.key_name}\tImagePath={result.custom['image_path']}",
                mtime=result.mtime,
            )
        )


class ServiceStartMode(Enum):
    AUTOMATIC = 2
    BOOT = 0
    DISABLED = 4
    MANUAL = 3
    SYSTEM = 1
    UNKNOWN = -1



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Gets the name of the last logged-on user"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        key = self.open_key(
            "Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI"
        )
        if not key:
            return

        for v in key.values():
            if (
                v.name().startswith("LastLoggedOn")
                and v.name() != "LastLoggedOnProvider"
            ):
                res = PluginResult(key=key, value=v)
                yield res

    def display_human(self, result):
        print(result.value_name, "\t", result.value_data)

    def display_machine(self, result):
        print(
            mactime(name=f"{result.value_name} {result.value_data}", mtime=result.mtime)
        )



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.portproxy import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("ControlSet001\\Services\\PortProxy\\v4tov4")
    reg = RegistryMock("SYSTEM", "system", key.root())
    reg.set_ccs(1)

    tcp = RegistryKeyMock("tcp", key)
    key.add_child(tcp)

    udp = RegistryKeyMock("udp", key)
    key.add_child(udp)

    https_proxy = RegistryValueMock("*/443", "127.0.0.1:1234", RegSZ)
    tcp.add_value(https_proxy)

    dns_proxy = RegistryValueMock("*/53", "8.8.8.8:80", RegSZ)
    udp.add_value(dns_proxy)

    return reg


def test_portproxy(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SYSTEM", "-")

    results = list(p.run())

    assert len(results) == 2, "There should be 2 results"

    for result in results:
        assert result.custom["proto"] in [
            "tcp",
            "udp",
        ], "The protocol should be either tcp or udp"

        if result.custom["proto"] == "tcp":
            assert result.value_name == "*/443", "The TCP portproxy FROM should be 443"
            assert (
                result.value_data == "127.0.0.1:1234"
            ), "The TCP portproxy TO should be 127.0.0.1:1234"
        else:  # UDP
            assert result.value_name == "*/53", "The UDP portproxy FROM should be 53"
            assert (
                result.value_data == "8.8.8.8:80"
            ), "The UDP portproxy TO should be 8.8.8.8:80"



# Plugin written by Tim Taylor, timtaylor3@yahoo.com
import struct
from Registry.RegistryParse import parse_windows_timestamp
from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Return the last shutdown time"""

    __REGHIVE__ = "SYSTEM"

    def run(self):

        key = self.open_key(self.get_currentcontrolset_path() + r"\Control\Windows")
        if not key:
            return

        for v in key.values():
            if v.name() == "ShutdownTime":
                binary = struct.unpack("<Q", v.value())[0]
                dt = parse_windows_timestamp(binary)
                value = dt.isoformat("T") + "Z"
                res = PluginResult(key=key, value=v)
                res.custom["LastShutdownTime"] = value
                yield res

    def display_human(self, result):
        print("Last Shutdown Time was: {0}".format(result.custom["LastShutdownTime"]))

    def display_machine(self, result):
        print(
            mactime(
                name=f"{result.value_name} {result.custom['LastShutdownTime']}",
                mtime=result.mtime,
            )
        )



import pytest
from Registry.Registry import RegDWord

from regrippy.plugins.printer_history import Plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)

PRINTERS = [
    "Microsoft Print To PDF",
    "Microsoft XPS Document Writer",
    "OneNote",
    "Fax",
    "Send To OneNote 2016",
    "MyEvilPrinter",
]


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(r"Printers\ConvertUserDevModesCount")
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    for v in PRINTERS:
        val = RegistryValueMock(v, 0x00000001, RegDWord)
        key.add_value(val)

    return reg


def test_printer_history(mock_reg):
    p = Plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == len(PRINTERS), f"There should be {len(PRINTERS)} results"

    for r in results:
        assert r.value_name in PRINTERS, "The printer name should be valid"



from regrippy import BasePlugin, PluginResult, mactime

# The entries in "ConvertUserDevModesCount" are not removed when a printer is deleted


class Plugin(BasePlugin):
    """Lists all printers that were connected to this machine"""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        k = self.open_key(r"Printers\ConvertUserDevModesCount")
        if not k:
            return

        for v in k.values():
            r = PluginResult(key=k, value=v)
            yield r

    def display_human(self, result):
        print(result.value_name)

    def display_machine(self, result):
        print(mactime(name=result.value_name, mtime=result.mtime))



import pytest
from Registry.Registry import RegBin, RegSZ

from regrippy.plugins.systeminfo import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_software():
    key = RegistryKeyMock.build("Microsoft\\Windows NT\\CurrentVersion")
    reg = RegistryMock("SOFTWARE", "software", key.root())

    key_install_date = RegistryKeyMock("InstallDate", key)
    key_owner = RegistryKeyMock("RegisteredOwner", key)
    key_product_name = RegistryKeyMock("ProductName", key)

    val1 = RegistryValueMock("InstallDate", 1523364100, RegBin)
    val2 = RegistryValueMock("RegisteredOwner", "Some Dude", RegSZ)
    val3 = RegistryValueMock("ProductName", "Windows 7 Professional", RegSZ)

    key.add_child(key_install_date)
    key.add_child(key_owner)
    key.add_child(key_product_name)

    key.add_value(val1)
    key.add_value(val2)
    key.add_value(val3)

    return reg


@pytest.fixture
def mock_control():
    host_name = RegistryKeyMock.build(
        "ControlSet001\\Control\\ComputerName\\ComputerName"
    )
    reg = RegistryMock("SYSTEM", "system", host_name.root())
    reg.set_ccs(1)

    val1 = RegistryValueMock("ComputerName", "TestPC", RegSZ)
    host_name.add_value(val1)

    current_control_set = reg.open("ControlSet001\\Control")
    current_control_set_window = RegistryKeyMock("Windows", current_control_set)
    current_control_set.add_child(current_control_set_window)

    val2 = RegistryValueMock("ShutdownTime", b"\xe40\xa5!\xed\xf7\xd3\x01", RegBin)
    current_control_set_window.add_value(val2)
    return reg


@pytest.fixture
def mock_services():
    interface1 = RegistryKeyMock.build(
        "ControlSet001\\Services\\Tcpip\\Parameters\\Interfaces\\{456A6A17D-21FC-123F-A689-A846D86EC008}"
    )
    reg = RegistryMock("SYSTEM", "system", interface1.root())
    reg.set_ccs(1)

    val1 = RegistryValueMock("DhcpIPAddress", "127.0.0.1", RegSZ)
    interface1.add_value(val1)

    interface2 = reg.open("ControlSet001\\Services\\Tcpip\\Parameters\\Interfaces")
    key_ip_address = (
        RegistryKeyMock.build("{123ee342-7039-466e-9d20-806e6f6e6963}\\IPAddress")
        .root()
        .open("{123ee342-7039-466e-9d20-806e6f6e6963}")
    )
    interface2.add_child(key_ip_address)

    val2 = RegistryValueMock("IPAddress", "127.0.0.2", RegSZ)
    key_ip_address.add_value(val2)

    return reg


def test_run(mock_software, mock_control, mock_services):
    # SOFTWARE
    p = plugin(mock_software, LoggerMock(), "SOFTWARE", "-")
    results = list(p.run())

    assert len(results) == 3, "There should be  3 results."
    assert (
        results[0].custom["value"] == "Install Date:\t\t2018-04-10T12:41:40Z"
    ), "It should have returned 'Install Date:\t\t2018-04-10T12:41:40Z'"
    assert (
        results[1].custom["value"] == "Registered Owner:\tSome Dude"
    ), "It should have returned 'Registered Owner\tSome Dude'"
    assert (
        results[2].custom["value"] == "Operating System:\tWindows 7 Professional"
    ), "It should have returned 'Operating System:\tWindows 7 Professional'"

    # SYSTEM Control path
    p = plugin(mock_control, LoggerMock(), "SYSTEM", "-")
    results = list(p.run())
    assert len(results) == 2, "There should be 2 results"
    assert (
        results[0].custom["value"] == "Hostname:\t\tTestPC"
    ), "It should have returned 'Hostname:\t\tTestPC'"
    assert (
        results[1].custom["value"] == "Last Shutdown Time:\t2018-05-30T08:06:36.766026Z"
    ), "It should have returned 'Install Date:\t2018-04-10T12:41:40Z'"

    # SYSTEM services
    p = plugin(mock_services, LoggerMock(), "SYSTEM", "-")
    results = list(p.run())
    assert len(results) == 2, "There should be 2 results"
    assert (
        results[0].custom["value"] == "IP Address:\t\t127.0.0.1"
    ), "It should have returned 'IP Address:\t127.0.0.1'"
    assert (
        results[1].custom["value"] == "IP Address:\t\t127.0.0.2"
    ), "It should have returned 'IP Address:\t\t127.0.0.2'"



from datetime import datetime

from regrippy import BasePlugin, PluginResult, mactime
from regrippy.thirdparty.ShimCacheParser import read_cache


class Plugin(BasePlugin):
    """Parse shim cache to show all executed binaries on machine"""

    __REGHIVE__ = "SYSTEM"

    def run(self):
        key = self.open_key(
            self.get_currentcontrolset_path()
            + r"\Control\Session Manager\AppCompatCache"
        ) or self.open_key(
            self.get_currentcontrolset_path()
            + r"\Control\Session Manager\AppCompatibility"
        )

        if not key:
            return

        for entry in read_cache(key.value("AppCompatCache").value()):
            res = PluginResult(key=key, value=None)
            res.custom["date"] = entry[0]
            if type(entry[2]) == bytes:
                res.custom["path"] = entry[2].decode("utf8")
            else:
                res.custom["path"] = entry[2]
            yield res

    def display_human(self, result):
        print(result.custom["date"] + "\t" + result.custom["path"])

    def display_machine(self, result):
        date = datetime.strptime(result.custom["date"], "%Y-%m-%d %H:%M:%S")
        atime = int(date.timestamp())

        print(mactime(name=result.custom["path"], mtime=result.mtime, atime=atime))



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Lists all printer ports setup on the machine"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        k = self.open_key(r"Microsoft\Windows NT\CurrentVersion\Ports")
        if not k:
            return

        for v in k.values():
            r = PluginResult(key=k, value=v)
            yield r

    def display_human(self, result):
        print(f"{result.value_name}: {result.value_data}")

    def display_machine(self, result):
        print(
            mactime(name=f"{result.value_name},{result.value_data}", mtime=result.mtime)
        )



from regrippy import BasePlugin, PluginResult, mactime
from Registry import Registry


class Plugin(BasePlugin):
    """Lists the recently connected to RDP servers"""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        key = self.open_key(r"Software\Microsoft\Terminal Server Client\Servers")
        if not key:
            return

        for subkey in key.subkeys():
            try:
                username = subkey.value("UsernameHint").value()
            except Registry.RegistryValueNotFoundException:
                username = ""

            res = PluginResult(key=subkey, value=None)
            res.custom["username"] = username
            yield res

    def display_human(self, result):
        print(f"{result.custom['username']}@{result.key_name}")

    def display_machine(self, result):
        print(
            mactime(
                name=f"{self.guess_username()} {result.custom['username']}@{result.key_name}",
                mtime=result.mtime,
            )
        )



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.lastloggedon import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        "Microsoft\\Windows\\CurrentVersion\\Authentication\\LogonUI"
    )
    reg = RegistryMock("SOFTWARE", "software", key.root())

    user = RegistryValueMock("LastLoggedOnUser", "MYDOMAIN\\User01", RegSZ)
    sam = RegistryValueMock("LastLoggedOnSAM", ".\\User02", RegSZ)
    provider = RegistryValueMock("LastLoggedOnProvider", "No", RegSZ)

    key.add_value(user)
    key.add_value(sam)
    key.add_value(provider)

    return reg


def test_lastloggedon(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SOFTWARE", "-")
    results = list(p.run())

    assert len(results) == 2, "There should be 2 results"
    assert set([r.value_name for r in results]) == set(
        ["LastLoggedOnUser", "LastLoggedOnSAM"]
    ), "The results should be User and SAM (no Provider)"



from regrippy import BasePlugin, PluginResult, mactime
from Registry.Registry import RegistryValueNotFoundException


class Plugin(BasePlugin):
    """List keyboard languages for the current user"""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        preload = self.open_key("Keyboard Layout\\Preload")
        substitutes = self.open_key("Keyboard Layout\\Substitutes")

        if not preload or not substitutes:
            return

        for preload_value in preload.values():
            preload_txt = preload_value.value()

            try:
                subs = substitutes.value(preload_txt)
            except RegistryValueNotFoundException:
                subs = None

            r = PluginResult(key=preload, value=preload_value)
            r.custom["substitute"] = subs
            yield r

    def display_human(self, result):
        subs_txt = self.get_subsitution_string(result.custom["substitute"])
        print(self.get_lang_name(result.value_data) + subs_txt)

    def display_machine(self, result):
        username = self.guess_username()
        subs_txt = self.get_subsitution_string(result.custom["substitute"])
        print(
            mactime(
                name=f"{username}\t{self.get_lang_name(result.value_data)}{subs_txt}",
                mtime=result.mtime,
            )
        )

    def get_lang_name(self, code):
        lang_code = int(code, 16) & (~0xDFF00000)
        if lang_code not in languageCodes.keys():
            return f"Unknown: {code}"
        else:
            return languageCodes[lang_code]

    def get_subsitution_string(self, value):
        subs_txt = ""
        if value:
            subs_txt = " / " + self.get_lang_name(value.value())
        return subs_txt


# Extracted from a Windows 10 1903 + data from Magnum DB
languageCodes = {
    0x00000401: "Arabic (101)",
    0x00000402: "Bulgarian (Typewriter)",
    0x00000404: "Chinese (Traditional) - US Keyboard",
    0x00000405: "Czech",
    0x00000406: "Danish",
    0x00000407: "German",
    0x00000408: "Greek",
    0x00000409: "US",
    0x0000040A: "Spanish",
    0x0000040B: "Finnish",
    0x0000040C: "French",
    0x0000040D: "Hebrew",
    0x0000040E: "Hungarian",
    0x0000040F: "Icelandic",
    0x00000410: "Italian",
    0x00000411: "Japanese",
    0x00000412: "Korean",
    0x00000413: "Dutch",
    0x00000414: "Norwegian",
    0x00000415: "Polish (Programmers)",
    0x00000416: "Portuguese (Brazilian ABNT)",
    0x00000418: "Romanian (Legacy)",
    0x00000419: "Russian",
    0x0000041A: "Standard",
    0x0000041B: "Slovak",
    0x0000041C: "Albanian",
    0x0000041D: "Swedish",
    0x0000041E: "Thai Kedmanee",
    0x0000041F: "Turkish Q",
    0x00000420: "Urdu",
    0x00000422: "Ukrainian",
    0x00000423: "Belarusian",
    0x00000424: "Slovenian",
    0x00000425: "Estonian",
    0x00000426: "Latvian",
    0x00000427: "Lithuanian IBM",
    0x00000428: "Tajik",
    0x00000429: "Persian",
    0x0000042A: "Vietnamese",
    0x0000042B: "Armenian Eastern",
    0x0000042C: "Azeri Latin",
    0x0000042E: "Sorbian Standard (Legacy)",
    0x0000042F: "Macedonian (FYROM)",
    0x00000432: "Setswana",
    0x00000437: "Georgian",
    0x00000438: "Faeroese",
    0x00000439: "Devanagari - INSCRIPT",
    0x0000043A: "Maltese 47-Key",
    0x0000043B: "Norwegian with Sami",
    0x0000043F: "Kazakh",
    0x00000440: "Kyrgyz Cyrillic",
    0x00000442: "Turkmen",
    0x00000444: "Tatar (Legacy)",
    0x00000445: "Bengali",
    0x00000446: "Punjabi",
    0x00000447: "Gujarati",
    0x00000448: "Oriya",
    0x00000449: "Tamil",
    0x0000044A: "Telugu",
    0x0000044B: "Kannada",
    0x0000044C: "Malayalam",
    0x0000044D: "Assamese - INSCRIPT",
    0x0000044E: "Marathi",
    0x00000450: "Mongolian Cyrillic",
    0x00000451: "Tibetan (PRC)",
    0x00000452: "United Kingdom Extended",
    0x00000453: "Khmer",
    0x00000454: "Lao",
    0x0000045A: "Syriac",
    0x0000045B: "Sinhala",
    0x0000045C: "Cherokee Nation",
    0x00000461: "Nepali",
    0x00000463: "Pashto (Afghanistan)",
    0x00000465: "Divehi Phonetic",
    0x00000468: "Hausa",
    0x0000046A: "Yoruba",
    0x0000046C: "Sesotho sa Leboa",
    0x0000046D: "Bashkir",
    0x0000046E: "Luxembourgish",
    0x0000046F: "Greenlandic",
    0x00000470: "Igbo",
    0x00000474: "Guarani",
    0x00000475: "Hawaiian",
    0x00000480: "Uyghur (Legacy)",
    0x00000481: "Maori",
    0x00000485: "Sakha",
    0x00000488: "Wolof",
    0x00000492: "Central Kurdish",
    0x00000804: "Chinese (Simplified) - US Keyboard",
    0x00000807: "Swiss German",
    0x00000809: "United Kingdom",
    0x0000080A: "Latin American",
    0x0000080C: "Belgian French",
    0x00000813: "Belgian (Period)",
    0x00000816: "Portuguese",
    0x0000081A: "Serbian (Latin)",
    0x0000082C: "Azeri Cyrillic",
    0x0000083B: "Swedish with Sami",
    0x00000843: "Uzbek Cyrillic",
    0x00000850: "Mongolian (Mongolian Script)",
    0x0000085D: "Inuktitut - Latin",
    0x0000085F: "Central Atlas Tamazight",
    0x00000C04: "Chinese (Traditional, Hong Kong S.A.R.) - US Keyboard",
    0x00000C0A: "Spanish (International)",  # MagnumDB
    0x00000C0C: "Canadian French (Legacy)",
    0x00000C1A: "Serbian (Cyrillic)",
    0x00000C51: "Dzongkha",
    0x00001004: "Chinese (Simplified, Singapore) - US Keyboard",
    0x00001009: "Canadian French",
    0x0000100C: "Swiss French",
    0x0000105F: "Tifinagh (Basic)",
    0x00001404: "Chinese (Traditional, Macao S.A.R.) - US Keyboard",
    0x00001809: "Irish",
    0x0000201A: "Bosnian (Cyrillic)",
    0x00004009: "India",
    0x00010401: "Arabic (102)",
    0x00010402: "Bulgarian (Latin)",
    0x00010405: "Czech (QWERTY)",
    0x00010407: "German (IBM)",
    0x00010408: "Greek (220)",
    0x00010409: "United States-Dvorak",
    0x0001040A: "Spanish Variation",
    0x0001040E: "Hungarian 101-key",
    0x00010410: "Italian (142)",
    0x00010415: "Polish (214)",
    0x00010416: "Portuguese (Brazilian ABNT2)",
    0x00010418: "Romanian (Standard)",
    0x00010419: "Russian (Typewriter)",
    0x0001041B: "Slovak (QWERTY)",
    0x0001041E: "Thai Pattachote",
    0x0001041F: "Turkish F",
    0x00010426: "Latvian (QWERTY)",
    0x00010427: "Lithuanian",
    0x0001042B: "Armenian Western",
    0x0001042C: "Azerbaijani (Standard)",
    0x0001042E: "Sorbian Extended",
    0x0001042F: "Macedonian (FYROM) - Standard",
    0x00010437: "Georgian (QWERTY)",
    0x00010439: "Hindi Traditional",
    0x0001043A: "Maltese 48-Key",
    0x0001043B: "Sami Extended Norway",
    0x00010444: "Tatar",
    0x00010445: "Bengali - INSCRIPT (Legacy)",
    0x00010451: "Tibetan (PRC) - Updated",
    0x00010453: "Khmer (NIDA)",
    0x0001045A: "Syriac Phonetic",
    0x0001045B: "Sinhala - Wij 9",
    0x0001045C: "Cherokee Nation Phonetic",
    0x0001045D: "Inuktitut - Naqittaut",
    0x00010465: "Divehi Typewriter",
    0x00010480: "Uyghur",
    0x0001080C: "Belgian (Comma)",
    0x0001083B: "Finnish with Sami",
    0x00010850: "Traditional Mongolian (Standard)",
    0x00010C00: "Myanmar (Phonetic order)",
    0x00011009: "Canadian Multilingual Standard",
    0x0001105F: "Tifinagh (Full)",
    0x00011809: "Gaelic",
    0x00020401: "Arabic (102) AZERTY",
    0x00020402: "Bulgarian (Phonetic)",
    0x00020405: "Czech Programmers",
    0x00020408: "Greek (319)",
    0x00020409: "United States-International",
    0x0002040D: "Hebrew (Standard)",
    0x00020418: "Romanian (Programmers)",
    0x00020419: "Russian - Mnemonic",
    0x0002041E: "Thai Kedmanee (non-ShiftLock)",
    0x00020422: "Ukrainian (Enhanced)",
    0x00020426: "Latvian (Standard)",
    0x00020427: "Lithuanian Standard",
    0x0002042B: "Armenian Phonetic",
    0x0002042E: "Sorbian Standard",
    0x00020437: "Georgian (Ergonomic)",
    0x00020445: "Bengali - INSCRIPT",
    0x00020449: "Tamil 99",
    0x0002083B: "Sami Extended Finland-Sweden",
    0x00020C00: "New Tai Lue",
    0x00030402: "Bulgarian",
    0x00030408: "Greek (220) Latin",
    0x00030409: "United States-Dvorak for left hand",
    0x0003041E: "Thai Pattachote (non-ShiftLock)",
    0x0003042B: "Armenian Typewriter",
    0x00030437: "Georgian Ministry of Education and Science Schools",
    0x00030C00: "Tai Le",
    0x00040402: "Bulgarian (Phonetic Traditional)",
    0x00040408: "Greek (319) Latin",
    0x00040409: "United States-Dvorak for right hand",
    0x00040437: "Georgian (Old Alphabets)",
    0x00040C00: "Ogham",
    0x00050408: "Greek Latin",
    0x00050409: "US English Table for IBM Arabic 238_L",
    0x00050429: "Persian (Standard)",
    0x00060408: "Greek Polytonic",
    0x00070C00: "Lisu (Basic)",
    0x00080C00: "Lisu (Standard)",
    0x00090C00: "N’Ko",
    0x000A0C00: "Phags-pa",
    0x000B0C00: "Buginese",
    0x000C0C00: "Gothic",
    0x000D0C00: "Ol Chiki",
    0x000E0C00: "Osmanya",
    0x000F0C00: "Old Italic",
    0x00100C00: "Sora",
    0x00110C00: "Javanese",
    0x00120C00: "Futhark",
    0x00130C00: "Myanmar (Visual Order)",
    0x00140C00: "ADLaM",
    0x00150C00: "Osage",
}



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.services import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("ControlSet002\\Services")
    reg = RegistryMock("SYSTEM", "system", key.root())
    reg.set_ccs(2)

    srv1 = RegistryKeyMock("Service1", key)
    key.add_child(srv1)
    srv1_image = RegistryValueMock(
        "ImagePath", "C:\\windows\\system32\\srv1.sys", RegSZ
    )
    srv1.add_value(srv1_image)

    srv2 = RegistryKeyMock("Service2", key)
    key.add_child(srv2)

    return reg


def test_services(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SYSTEM", "-")

    results = list(p.run())

    assert len(results) == 2, "There should be two results"

    assert (
        results[0].key_name == "Service1"
    ), "The first service should be named 'Service1'"
    assert (
        results[0].custom["image_path"] == "C:\\windows\\system32\\srv1.sys"
    ), "The first service's image path should match"

    assert (
        results[1].key_name == "Service2"
    ), "The second service should be named 'Service2'"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.uninstall import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("Microsoft\\Windows\\CurrentVersion\\Uninstall")
    reg = RegistryMock("SOFTWARE", "software", key.root())

    word = RegistryKeyMock("winword", key)
    key.add_child(word)
    word_name = RegistryValueMock("DisplayName", "Microsoft Office Word", RegSZ)
    word_uninstall = RegistryValueMock(
        "UninstallString", "C:\\Program Files\\Word\\uninstall.exe", RegSZ
    )
    word.add_value(word_name)
    word.add_value(word_uninstall)

    mlwr = RegistryKeyMock("tasksche", key)
    key.add_child(mlwr)
    mlw_uninstall = RegistryValueMock(
        "UninstallString", "C:\\boguspath\\uninstall.exe", RegSZ
    )
    mlwr.add_value(mlw_uninstall)

    return reg


def test_uninstall(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SOFTWARE", "-")

    results = list(p.run())

    assert len(results) == 2, "There should be 2 results"

    assert (
        results[0].key_name == "winword"
    ), "The first result keyname should be winword"
    assert (
        results[0].custom["display_name"] == "Microsoft Office Word"
    ), "The first result nice name should be Word"
    assert (
        results[0].custom["uninstall_string"]
        == "C:\\Program Files\\Word\\uninstall.exe"
    ), "The first result's uninstall path should match"

    assert (
        results[1].key_name == "tasksche"
    ), "The second result should be the bogus 'tasksche' entry"
    assert (
        results[1].custom["uninstall_string"] == "C:\\boguspath\\uninstall.exe"
    ), "The second entry's uninstall path should match"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.mndmru import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Map Network Drive MRU"
    )
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    mrulist = RegistryValueMock("MRUList", "ab", RegSZ)

    a = RegistryValueMock("a", "first", RegSZ)
    b = RegistryValueMock("b", "second", RegSZ)

    key.add_value(mrulist)
    key.add_value(b)
    key.add_value(a)

    return reg


def test_mndmru(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == 2, "There should be 2 results"
    assert results[0].value_data == "first", "First result should be 'a'"
    assert results[1].value_data == "second", "Second result should be 'b'"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.filedialogmru import Plugin as plugin

from .reg_mock import LoggerMock, RegistryKeyMock, RegistryMock, RegistryValueMock


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\ComDlg32"
    )
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    # Open / Save
    opensavemru = RegistryKeyMock("OpenSaveMRU", key)
    key.add_child(opensavemru)

    osmru_list = RegistryValueMock("MRUList", "bar", RegSZ)
    osmru_b = RegistryValueMock("b", "evil.bat", RegSZ)
    osmru_a = RegistryValueMock("a", "LOVE-LETTER-FOR-YOU.TXT.vbs", RegSZ)
    osmru_r = RegistryValueMock("r", "quarterly_reports.docx", RegSZ)

    opensavemru.add_value(osmru_list)
    opensavemru.add_value(osmru_a)
    opensavemru.add_value(osmru_b)
    opensavemru.add_value(osmru_r)

    # Last visited
    lastvisitedmru = RegistryKeyMock("LastVisitedMRU", key)
    key.add_child(lastvisitedmru)

    lvmru_list = RegistryValueMock("MRUList", "ab", RegSZ)
    lvmru_a = RegistryValueMock("a", "%USERPROFILE%\\Desktop", RegSZ)
    lvmru_b = RegistryValueMock("b", "C:\\Temp", RegSZ)

    lastvisitedmru.add_value(lvmru_list)
    lastvisitedmru.add_value(lvmru_a)
    lastvisitedmru.add_value(lvmru_b)

    return reg


def test_filedialogmru(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == 5, "There should be 5 results"

    assert results[0].value_data in [
        "evil.bat",
        "%USERPROFILE%\\Desktop",
    ], "The first result should be either from OpenSave or LastVisited"

    if results[0].value_data == "evil.bat":
        # OpenSaveMRU
        assert (
            results[1].value_data == "LOVE-LETTER-FOR-YOU.TXT.vbs"
        ), "The second result should be 'a'"
        assert (
            results[2].value_data == "quarterly_reports.docx"
        ), "The third result should be 'r'"

        # Then LastVisitedMRU
        assert (
            results[3].value_data == "%USERPROFILE%\\Desktop"
        ), "The fourth result shoul be 'a'"
        assert results[4].value_data == "C:\\Temp", "The fifth result shoul be 'b'"
    else:
        # LastVisitedMRU
        assert results[1].value_data == "C:\\Temp", "The second result shoul be 'b'"

        # Then OpenSaveMRU
        assert results[2].value_data == "evil.bat", "The third result shoul be 'b'"
        assert (
            results[3].value_data == "LOVE-LETTER-FOR-YOU.TXT.vbs"
        ), "The fourth result should be 'a'"
        assert (
            results[4].value_data == "quarterly_reports.docx"
        ), "The fifth result should be 'r'"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.rdphint import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("Software\\Microsoft\\Terminal Server Client\\Servers")
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    srv1 = RegistryKeyMock("server1.com", key)
    srv2 = RegistryKeyMock("server2.com", key)

    key.add_child(srv1)
    key.add_child(srv2)

    srv1username = RegistryValueMock("UsernameHint", "root", RegSZ)
    srv1.add_value(srv1username)

    return reg


def test_rdphint(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == 2, "There should only be 2 results"
    assert results[0].key_name == "server1.com", "First server should be server1.com"
    assert results[0].custom["username"] == "root", "First server should have user hint"
    assert results[1].key_name == "server2.com", "Second server should be server2.com"
    assert results[1].custom["username"] == "", "Second server should not have username"



from regrippy import BasePlugin, PluginResult


class Plugin(BasePlugin):
    """Get the Windows version"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        key = self.open_key("Microsoft\\Windows NT\\CurrentVersion")
        if not key:
            return

        value = key.value("ProductName")
        yield PluginResult(key=key, value=value)

    def display_human(self, result):
        print(result.value_data)



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Reads Terminal Client (aka mstsc.exe) Most Recently Used address"""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        key = self.open_key(r"Software\Microsoft\Terminal Server Client\Default")
        if not key or not key.values():
            return

        for value in key.values():
            res = PluginResult(key=key, value=value)
            yield res

    def display_human(self, result):
        print(result.value_data)

    def display_machine(self, result):
        print(
            mactime(
                name=f"{self.guess_username()}\t{result.value_data}", mtime=result.mtime
            )
        )



import pytest
from Registry.Registry import RegDWord, RegSZ

from regrippy.plugins.antivirus import Plugin as plugin

from .reg_mock import LoggerMock, RegistryKeyMock, RegistryMock, RegistryValueMock


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        r"Microsoft\Security Center\Provider\Av\{D68DDC3A-831F-4fae-9E44-DA132C1ACF46}"
    )
    reg = RegistryMock("SOFTWARE", "software", key.root())

    key.add_value(RegistryValueMock("DISPLAYNAME", "Antivirus Windows Defender", RegSZ))
    key.add_value(
        RegistryValueMock("GUID", "{D68DDC3A-831F-4fae-9E44-DA132C1ACF46}", RegSZ)
    )
    key.add_value(RegistryValueMock("PRODUCTEXE", "windowsdefender://", RegSZ))
    key.add_value(
        RegistryValueMock(
            "REPORTINGEXE", "%ProgramFiles%\\Windows Defender\\MsMpeng.exe", RegSZ
        )
    )
    key.add_value(RegistryValueMock("STATE", 0x61100, RegDWord))

    return reg


def test_antivirus(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SOFTWARE", "-")

    results = list(p.run())

    assert len(results) == 1, "There should be a single result"
    assert (
        results[0].custom["DisplayName"] == "Antivirus Windows Defender"
    ), "The extracted antivirus name must be 'Antivirus Windows Defender'"
    assert (
        results[0].custom["State"] == "ON | UP_TO_DATE | MICROSOFT_PRODUCT"
    ), "The state must match the following: enabled, up-to-date Microsoft antivirus"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.mstscmru import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(r"Software\Microsoft\Terminal Server Client\Default")
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    mru0 = RegistryValueMock("MRU0", "127.0.0.1", RegSZ)
    mru1 = RegistryValueMock("MRU1", "10.0.0.1", RegSZ)

    key.add_value(mru0)
    key.add_value(mru1)

    return reg


def test_mndmru(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == 2, "There should be 2 results"
    assert results[0].value_data == "127.0.0.1", "First result should be '127.0.0.1'"
    assert results[1].value_data == "10.0.0.1", "Second result should be '10.0.0.1'"



import pytest
from Registry.Registry import RegBin, RegSZ

from regrippy.plugins.recentdocs import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\RecentDocs"
    )
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    mrulistex = RegistryValueMock("MRUListEx", b"random bytes", RegBin)
    key.add_value(mrulistex)

    document = RegistryValueMock(
        "a",
        b"m\x00y\x00d\x00o\x00c\x00u\x00m\x00e\x00n\x00t\x00.\x00d\x00o\x00c\x00x\x00\x00random_bytes\xCA\xFE",
        RegBin,
    )
    key.add_value(document)

    return reg


def test_recentdocs(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    assert len(results) == 1, "There should be a single result"
    assert (
        results[0].custom["docname"] == "mydocument.docx"
    ), "The document should be named 'mydocument.docx'"



import pytest
from Registry.Registry import RegBin

from regrippy.plugins.localusers import Plugin as plugin

from .reg_mock import LoggerMock, RegistryKeyMock, RegistryMock


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("SAM\\Domains\\Account\\Users\\Names")
    reg = RegistryMock("SAM", "sam", key.root())

    user1 = RegistryKeyMock("JohnDoe", key)
    user2 = RegistryKeyMock("JaneDoe", key)

    key.add_child(user1)
    key.add_child(user2)

    return reg


def test_localusers(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SAM", "-")

    results = list(p.run())
    assert len(results) == 2, "There should be two results"

    assert any(
        [x.key_name == "JohnDoe" for x in results]
    ), "There should be a user named 'JohnDoe'"
    assert any(
        [x.key_name == "JaneDoe" for x in results]
    ), "There should be a user named 'JaneDoe'"



from regrippy import BasePlugin, mactime, PluginResult
from Registry.Registry import RegistryValueNotFoundException


class Plugin(BasePlugin):
    """Extracts the user's proxy settings"""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        k = self.open_key(
            r"Software\Microsoft\Windows\CurrentVersion\Internet Settings"
        )
        if not k:
            return

        r = PluginResult(key=k)

        r.custom["type"] = "proxy"
        r.custom["enabled"] = self.safe_value(k, "ProxyEnabled", 0) == 1
        r.custom["proxy"] = self.safe_value(k, "ProxyServer", "N/A")
        r.custom["exceptions"] = self.safe_value(k, "ProxyOverride", "")

        yield r

        r = PluginResult(key=k)
        r.custom["type"] = "autoconfig"
        r.custom["proxypac"] = self.safe_value(k, "AutoConfigURL", "N/A")

        yield r

    def display_human(self, r):
        if r.custom["type"] == "proxy":
            print("[PROXY]")
            print(f"\tEnabled: {r.custom['enabled']}")
            print(f"\tProxy URL: {r.custom['proxy']}")
            print(f"\tExceptions: {r.custom['exceptions']}")
        elif r.custom["type"] == "autoconfig":
            print("[AUTO-CONFIG]")
            print(f"\tProxyPac URL: {r.custom['proxypac']}")
        print()

    def display_machine(self, r):
        print(mactime(name="Proxy settings modified", mtime=r.mtime))

    def safe_value(self, k, name, default):
        for v in k.values():
            if v.name() == name:
                return v.value()
        return default



import re

from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """get all KB update installation status"""

    __REGHIVE__ = "SOFTWARE"

    STATUS_CODES = {
        0x0: "Absent",
        0x5: "Uninstall pending",
        0x10: "Resolving",
        0x20: "Resolved",
        0x30: "Staging",
        0x40: "Staged",
        0x50: "Superseded",
        0x60: "Install pending",
        0x65: "Partially installed",
        0x70: "Installed",
        0x80: "Permanent",
    }

    def run(self):
        k = self.open_key(
            r"Microsoft\Windows\CurrentVersion\Component Based Servicing\Packages"
        )
        if not k:
            return

        r = r"Package(_(?P<pkgnum>[0-9]+))?_for_(?P<kb>KB[0-9]+)"
        for subkey in k.subkeys():
            match = re.match(r, subkey.name())
            if not match:
                continue

            res = PluginResult(key=subkey)
            res.custom["kb"] = match.group("kb")
            res.custom["number"] = match.group("pkgnum")
            res.custom["status_code"] = subkey.value("CurrentState").value()
            res.custom["status"] = self.STATUS_CODES.get(
                res.custom["status_code"], "unknown"
            )

            yield res

    def display_human(self, res):
        if res.custom["number"]:
            print(
                f"{res.custom['kb']} (#{res.custom['number']}): {res.custom['status']} (0x{res.custom['status_code']:02x})"
            )
        else:
            print(
                f"{res.custom['kb']}: {res.custom['status']} (0x{res.custom['status_code']:02x})"
            )

    def display_machine(self, res):
        if res.custom["number"]:
            print(
                mactime(
                    mtime=res.mtime,
                    name=f"{res.custom['kb']} #{res.custom['number']} [{res.custom['status']}]",
                )
            )
        else:
            print(
                mactime(
                    mtime=res.mtime, name=f"{res.custom['kb']} [{res.custom['status']}]"
                )
            )



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Reads OpenSaveMRU and LastVisitedMRU keys (Most Recently Used files in Save As / Open file dialogs)"""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        for path in [
            r"Software\Microsoft\Windows\CurrentVersion\Explorer\ComDlg32\OpenSaveMRU",
            r"Software\Microsoft\Windows\CurrentVersion\Explorer\ComDlg32\LastVisitedMRU",
        ]:
            key = self.open_key(path)
            if not key or not key.values():
                continue

            order = key.value("MRUList").value()
            values = dict(
                [(v.name(), v) for v in key.values() if v.name() != "MRUList"]
            )

            for letter in order:
                yield PluginResult(key=key, value=values[letter])

    def display_human(self, result):
        print(result.value_data)

    def display_machine(self, result):
        print(
            mactime(
                name=f"{self.guess_username()}\t{result.value_data}", mtime=result.mtime
            )
        )



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Reads startup programs from various hives"""

    __REGHIVE__ = ["NTUSER.DAT", "SOFTWARE"]

    def run(self):
        if self.hive_name == "NTUSER.DAT":
            paths = [
                r"Software\Microsoft\Windows\CurrentVersion\Run",
                r"Software\Microsoft\Windows\CurrentVersion\RunOnce",
                r"Software\Microsoft\Windows NT\CurrentVersion\Windows\Run",
            ]
        else:  # SOFTWARE
            paths = [
                r"Microsoft\Windows\CurrentVersion\Run",
                r"Microsoft\Windows\CurrentVersion\RunOnce",
                r"Microsoft\Windows\CurrentVersion\Policies\Explorer\Run",
            ]

        for path in paths:
            key = self.open_key(path)
            if not key:
                continue

            for v in key.values():
                res = PluginResult(key=key, value=v)
                yield res

    def display_human(self, result):
        print(result.value_name, "//", result.value_data)

    def display_machine(self, result):
        print(
            mactime(
                name=f"{result.value_name}\t{result.value_data}", mtime=result.mtime
            )
        )



from regrippy import BasePlugin, PluginResult


class Plugin(BasePlugin):
    """gets the default antivirus, and retrieves the status of all installed antiviruses"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        k = self.open_key(r"Microsoft\Security Center\Provider\Av")
        if not k:
            return

        for subkey in k.subkeys():
            r = PluginResult(key=subkey)
            r.custom = {
                "DisplayName": subkey.value("DISPLAYNAME").value(),
                "ProductExe": subkey.value("PRODUCTEXE").value(),
                "ReportingExe": subkey.value("REPORTINGEXE").value(),
                "State": " | ".join(self.parse_state(subkey.value("STATE").value())),
            }
            yield r

    def display_human(self, r):
        print(f"{r.custom['DisplayName']}")
        print(f"\t{r.custom['ProductExe']}")
        print(f"\t{r.custom['State']}")
        print()

    def parse_state(self, state):
        # Source: https://mcpforlife.com/2020/04/14/how-to-resolve-this-state-value-of-av-providers/
        enums = []

        if state & 0x3000 == 0x3000:
            enums.append("EXPIRED")
        elif state & 0x2000 == 0x2000:
            enums.append("SNOOZED")
        elif state & 0x1000 == 0x1000:
            enums.append("ON")
        else:
            enums.append("OFF")

        if state & 0x10 == 0x10:
            enums.append("OUT_OF_DATE")
        else:
            enums.append("UP_TO_DATE")

        if state & 0x100 == 0x100:
            enums.append("MICROSOFT_PRODUCT")
        else:
            enums.append("NON_MICROSOFT_PRODUCT")

        return enums



from regrippy import BasePlugin, PluginResult


class Plugin(BasePlugin):
    """Produces a timeline of every key's last modification date"""

    __REGHIVE__ = "ALL"

    def run(self):
        key = self.reg.root()

        yield from self.dump(key)

    def dump(self, key):
        res = PluginResult(key=key)
        res.path = self.cleanup_path(res.path)
        yield res

        for subkey in key.subkeys():
            yield from self.dump(subkey)

    def cleanup_path(self, s):
        parts = s.split("\\")[1:]
        hive_path = self.reg.hive_name()
        hive_name = hive_path.split("\\")[-1]
        return hive_name + "\\" + "\\".join(parts)



import pytest
from Registry.Registry import RegBin

from regrippy.plugins.shimcache import Plugin as plugin
from tests.reg_mock import RegistryValueMock

from .reg_mock import LoggerMock, RegistryKeyMock, RegistryMock

_entry_win10_ = (
    b"\x00" * 0x30
    + b"10ts"
    + b"\x00\x00\x00\x00\x2a\x01\x00\x00\x1c\x00t\x00e\x00s\x00t\x00_\x00s\x00h\x00i\x00m\x00c\x00a\x00c\x00h\x00e\x00\x1f\xb1\xb8\x6f\xaf\xf5\xd4\x01"
)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(
        "ControlSet001\\Control\\Session Manager\\AppCompatCache"
    )
    reg = RegistryMock("SYSTEM", "system", key.root())
    reg.set_ccs(1)

    val = RegistryValueMock("AppCompatCache", _entry_win10_, RegBin)
    key.add_value(val)

    return reg


def test_shimcache(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SYSTEM", "-")

    results = list(p.run())

    assert len(results) == 1, "There should only be 1 results"
    assert results[0].custom["path"] == "test_shimcache"



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.run import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_software():
    curver_run = RegistryKeyMock.build("Microsoft\\Windows\\CurrentVersion\\Run")
    reg = RegistryMock("SOFTWARE", "software", curver_run.root())

    curver = reg.open("Microsoft\\Windows\\CurrentVersion")
    curver_runonce = RegistryKeyMock("RunOnce", curver)
    curver.add_child(curver_runonce)

    policies = RegistryKeyMock.build("Policies\\Explorer\\Run").root().open("Policies")
    curver.add_child(policies)

    policies_run = reg.open(
        "Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer\\Run"
    )

    # Add a sample entry in each key
    for k in [curver_run, curver_runonce, policies_run]:
        v = RegistryValueMock("autorun", "evil.exe", RegSZ)
        k.add_value(v)

    return reg


@pytest.fixture
def mock_ntuser():
    curver_run = RegistryKeyMock.build(
        "Software\\Microsoft\\Windows\\CurrentVersion\\Run"
    )
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", curver_run.root())

    curver = reg.open("Software\\Microsoft\\Windows\\CurrentVersion")
    curver_runonce = RegistryKeyMock("RunOnce", curver)
    curver.add_child(curver_runonce)

    winnt = (
        RegistryKeyMock.build("Windows NT\\CurrentVersion\\Windows\\Run")
        .root()
        .open("Windows NT")
    )
    reg.open("Software\\Microsoft").add_child(winnt)

    winnt_run = reg.open(
        "Software\\Microsoft\\Windows NT\\CurrentVersion\\Windows\\Run"
    )

    # Add a sample entry in each key
    for k in [curver_run, curver_runonce, winnt_run]:
        v = RegistryValueMock("autorun", "evil.exe", RegSZ)
        k.add_value(v)

    return reg


def test_run(mock_software, mock_ntuser):
    # SOFTWARE
    p = plugin(mock_software, LoggerMock(), "SOFTWARE", "-")
    results = list(p.run())

    assert len(results) == 3, "There should be 3 results, one per key"
    for r in results:
        assert (
            r.value_name == "autorun"
        ), f"Program label for {r.key_path} should be autorun"
        assert (
            r.value_data == "evil.exe"
        ), f"Program path for {r.keypath} should be evil.exe"

    # NTUSER.DAT
    p = plugin(mock_ntuser, LoggerMock(), "NTUSER.DAT", "-")
    results = list(p.run())

    assert len(results) == 3, "There should be 3 results, one per key"
    for r in results:
        assert (
            r.value_name == "autorun"
        ), f"Program label for {r.key_path} should be autorun"
        assert (
            r.value_data == "evil.exe"
        ), f"Program path for {r.keypath} should be evil.exe"



import platform
import subprocess
import sys
import os


if __name__ == "__main__":
    # Generate "testcoverage.rst"
    parent_dir = os.path.join(os.path.dirname(sys.argv[0]), "..")
    coverage = subprocess.check_output(
        ["python", "tests/get_coverage.py"], cwd=parent_dir
    )

    with open("testcoverage.rst", "wb") as f:
        f.write(coverage)

    # Run "make dirhtml"
    command = "make"
    if platform.system() == "Windows":
        command = "make.bat"

    proc = subprocess.Popen([command, "dirhtml"])
    proc.wait()

    os.unlink("testcoverage.rst")

    sys.exit(proc.returncode)



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.putty import Plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("Software\\SimonTatham\\PuTTY")
    reg = RegistryMock("NTUSER.DAT", "ntuser.dat", key.root())

    sessions = RegistryKeyMock("Sessions", key)
    key.add_child(sessions)

    # Sample session to my-ssh-server.com
    sess_key = RegistryKeyMock("MySSH", sessions)
    sessions.add_child(sess_key)

    sess_key.add_value(RegistryValueMock("HostName", "my-ssh-server.com", RegSZ))
    sess_key.add_value(RegistryValueMock("Protocol", "ssh", RegSZ))

    # Host key to evil-server.eu
    ssh_hosts = RegistryKeyMock("SshHostKeys", key)
    key.add_child(ssh_hosts)

    ssh_hosts.add_value(
        RegistryValueMock("ecdsa@22:evil-server.eu", "0xbadc00ffee", RegSZ)
    )

    return reg


def test_putty(mock_reg):
    p = Plugin(mock_reg, LoggerMock(), "NTUSER.DAT", "-")

    results = list(p.run())

    sessions = [x for x in results if x.custom["type"] == "session"]
    host_keys = [x for x in results if x.custom["type"] == "sshkey"]

    assert len(sessions) == 1, "There should be a single saved session"
    assert len(host_keys) == 1, "There should be a single saved SSH key"

    session = sessions[0]
    host_key = host_keys[0]

    assert session.key_name == "MySSH", "The session's name should be 'MySSH'"
    assert (
        session.custom["host"] == "my-ssh-server.com"
    ), "The session should point to my-ssh-server.com"
    assert session.custom["protocol"] == "ssh", "The session should be using SSH"

    assert (
        host_key.custom["crypto"] == "ecdsa"
    ), "Host key should be detected as 'ecdsa'"
    assert (
        host_key.custom["host"] == "22:evil-server.eu"
    ), "Host for the saved key should be 'evil-server.eu'"



from regrippy import BasePlugin, PluginResult, mactime
from Registry import Registry


class Plugin(BasePlugin):
    """Lists installed software"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        key = self.open_key("Microsoft\\Windows\\CurrentVersion\\Uninstall")
        if not key:
            return

        for program in key.subkeys():
            try:
                display_name = program.value("DisplayName").value()
            except Registry.RegistryValueNotFoundException:
                display_name = "[N/A]"

            try:
                uninstall_string = program.value("UninstallString").value()
            except Registry.RegistryValueNotFoundException:
                uninstall_string = "[N/A]"

            res = PluginResult(key=program)
            res.custom["display_name"] = display_name
            res.custom["uninstall_string"] = uninstall_string
            yield res

    def display_human(self, result):
        print(
            f"{result.key_name} ({result.custom['display_name']})\n\t{result.custom['uninstall_string']}\n"
        )

    def display_machine(self, result):
        print(
            mactime(
                name=f"{result.key_name};{result.custom['display_name']};{result.custom['uninstall_string']}",
                mtime=result.mtime,
            )
        )



import pytest
from Registry.Registry import RegBin, RegSZ

from regrippy.plugins.version import Plugin as plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build("Microsoft\\Windows NT\\CurrentVersion")
    reg = RegistryMock("SOFTWARE", "software", key.root())

    name = RegistryValueMock("ProductName", "ENCOM OS-12", RegSZ)
    key.add_value(name)

    return reg


def test_version(mock_reg):
    p = plugin(mock_reg, LoggerMock(), "SOFTWARE", "-")

    results = list(p.run())

    assert len(results) == 1, "There should be a single result"
    assert results[0].value_data == "ENCOM OS-12", "The retrieved version should match"



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Reads 'Map Network Drive MRU' key (Most Recently Used remote drives)"""

    __REGHIVE__ = "NTUSER.DAT"

    def run(self):
        key = self.open_key(
            r"Software\Microsoft\Windows\CurrentVersion\Explorer\Map Network Drive MRU"
        )
        if not key or not key.values():
            return

        order = key.value("MRUList")
        values = dict([(v.name(), v) for v in key.values() if v.name() != "MRUList"])

        for letter in order.value():
            res = PluginResult(key=key, value=values[letter])
            yield res

    def display_human(self, result):
        print(result.value_data)

    def display_machine(self, result):
        print(
            mactime(
                name=f"{self.guess_username()}\t{result.value_data}", mtime=result.mtime
            )
        )



from datetime import datetime

from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Reads TeamViewer configuration"""

    __REGHIVE__ = "SOFTWARE"

    def run(self):
        path = r"TeamViewer"

        k = self.open_key(path)
        if k:
            yield from self.process_key(k)

        k = self.open_key("Wow6432Node\\" + path)
        if k:
            yield from self.process_key(k)

    def process_key(self, k):
        r = PluginResult(key=k)

        values = [x.name() for x in k.values()]

        r.custom = {
            "AlwaysOnline": k.value("Always_Online").value() == 1,
            "ClientID": k.value("ClientID").value(),
            "LastStartupTime": k.value("LastStartupTime")
            if "LastStartupTime" in values
            else -1,
            "Version": k.value("Version").value(),
        }
        yield r

    def display_human(self, r):
        if "wow6432node" in r.path.lower():
            print("[32-bit application]")

        print(f"Always Online: {r.custom['AlwaysOnline']}")
        print(f"Client ID: {r.custom['ClientID']}")
        print(
            f"LastStartupTime: {r.custom['LastStartupTime']} ({datetime.fromtimestamp(r.custom['LastStartupTime'])})"
        )

    def display_machine(self, r):
        print(
            mactime(
                mtime=r.custom["LastStartupTime"],
                name=f"TeamViewer startup ({r.custom['ClientID']})",
            )
        )



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Extract the local user list from the SAM database"""

    __REGHIVE__ = "SAM"

    def run(self):
        k = self.open_key("SAM\\Domains\\Account\\Users\\Names")
        if not k:
            return

        for user in k.subkeys():
            res = PluginResult(key=user, value=None)
            yield res

    def display_human(self, res):
        print(res.key_name)

    def display_machine(self, res):
        print(mactime(name=res.key_name, mtime=res.mtime))



import struct

from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Extract local group information from the SAM & SOFTWARE databases"""

    # In this case, order matters. The plugins needs the SOFTWARE hive to be parsed first to preperly enrich results.
    __REGHIVE__ = ["SOFTWARE", "SAM"]

    # Use class variable to persist across runs
    user_profile_list = []

    def sid2asc(self, bin_sid):
        """Turns a binary SID into its ASCII representation"""
        # SID structure:
        #   + 1 byte (revision, little endian)
        #   + 1 byte (count of sub auths, little endian)
        #   + 6 bytes (authority value, big endian)
        #   + n * 4 bytes where n is the count of sub auths

        # Construct the fixed-length part of the SID
        rev, sub_auth_count, bytes_idauth = struct.unpack("<BB6s", bin_sid[:8])
        int_idauth = int.from_bytes(bytes_idauth, byteorder="big", signed=False)
        str_sid = "S-{0}-{1}".format(rev, int_idauth)

        # Get the variable part and iterate over each sub auth (4-byte little-endian integer)
        bin_sub_auths = bin_sid[8:]
        for i in range(0, sub_auth_count * 4, 4):
            str_sid += "-{0}".format(struct.unpack("<L", bin_sub_auths[i : i + 4])[0])

        return str_sid

    def machine_sid(self):
        """Gets the machine SID from the SAM hive"""
        # The machine SID on which all local accounts are based sits at the end of the Account key's V data
        key_machine = self.open_key("SAM\\Domains\\Account")
        v_raw = key_machine.value("V").value()
        v_data = v_raw[len(v_raw) - 24 :]
        return self.sid2asc(v_data)

    def user_sids_sam(self):
        """Gets user SIDs from the SAM hive"""
        machine_sid = self.machine_sid()
        key_accounts = self.open_key("SAM\\Domains\\Account\\Users\\Names")
        if not key_accounts:
            return
        for subkey in key_accounts.subkeys():
            # User SIDs are built by appending their Relative ID to the machine SID.
            # Relative IDs are stored inside Names' subkeys; each has a default value where the value *type* holds the
            # integer value of the RID.
            sid = "{0}-{1}".format(machine_sid, subkey.value("(default)").value_type())
            # User names are stored inside Names' subkeys; each subkey's name holds the associated user name.
            name = subkey.name()
            if not any(
                profile.get("sid", None) == sid for profile in self.user_profile_list
            ):
                self.user_profile_list.append({"sid": sid, "name": name})
        return

    def user_sids_soft(self):
        """Gets user SIDs from the SOFTWARE hive"""
        key_profiles = self.open_key(
            "Microsoft\\Windows NT\\CurrentVersion\\ProfileList"
        )
        if not key_profiles:
            return
        for subkey in key_profiles.subkeys():
            # User SIDs are stored inside ProfileList's subkeys; each subkey's name holds the associated user SID.
            sid = subkey.name()
            # The "profile path" points to the user folder; we can use it to map the SID to the user
            name = subkey.value("ProfileImagePath").value().rpartition("\\")[2]
            if not any(
                profile.get("sid", None) == sid for profile in self.user_profile_list
            ):
                self.user_profile_list.append({"sid": sid, "name": name})
        return

    def yield_groups(self):
        """Yields a dictionary representation of groups and their members, from the SAM hive"""
        key_groups = self.open_key("SAM\\Domains\\Builtin\\Aliases")
        if not key_groups:
            return

        # Yield a custom structure with information from the SAM hive
        for subkey in key_groups.subkeys():
            if subkey.name().startswith("00000"):
                c_raw = subkey.value("C").value()
                # Header is 13 little-endian unsigned long values and holds among others:
                # - Group's name offset and length
                # - Group's description offset and length
                # - Group members' offset and size (as binary SIDs)
                c_header = struct.unpack("<13L", c_raw[:0x34])
                c_data = c_raw[0x34:]
                c_gname_off = c_header[4]
                c_gname_len = c_header[5]
                c_gdesc_off = c_header[7]
                c_gdesc_len = c_header[8]
                c_gmembers_off = c_header[10]
                c_gmembers_size = c_header[12]

                member_sid_list = []
                off = c_gmembers_off
                for _ in range(c_gmembers_size):
                    # 1 member == 1 binary SID
                    # SID structure:
                    #   + 1 byte (revision, little endian)
                    #   + 1 byte (count of sub auths, little endian)
                    #   + 6 bytes (authority value, big endian)
                    #   + n * 4 bytes where n is the count of sub auths
                    # Look ahead to check the count of sub authorities to compute SID size
                    sub_auth_count = struct.unpack("<B", c_data[off + 1 : off + 2])[0]
                    sid_size = 8 + sub_auth_count * 4
                    member_sid_list.append(self.sid2asc(c_data[off : off + sid_size]))
                    off += sid_size

                res = PluginResult(key=subkey, value=None)
                res.custom = {
                    "name": c_data[c_gname_off : c_gname_off + c_gname_len].decode(
                        "utf-16-le"
                    ),
                    "desc": c_data[c_gdesc_off : c_gdesc_off + c_gdesc_len].decode(
                        "utf-16-le"
                    ),
                    "size": c_gmembers_size,
                    "member_sids": member_sid_list,
                }
                yield res

    def run(self):
        if self.hive_name == "SOFTWARE":
            self.user_sids_soft()
        elif self.hive_name == "SAM":
            self.user_sids_sam()
            yield from self.yield_groups()

    def display_human(self, res):
        group = res.custom
        print('"{0}" with {1} direct members'.format(group["name"], group["size"]))
        if group["size"] > 0:
            for member_sid in group["member_sids"]:
                mapped_user = ""
                for user in self.user_profile_list:
                    if user["sid"] == member_sid:
                        mapped_user = user["name"]
                if mapped_user != "":
                    print("\t{0} ({1})".format(member_sid, mapped_user))
                else:
                    print("\t{0}".format(member_sid))

    def display_machine(self, res):
        print(mactime(name=res.custom["name"], mtime=res.mtime))



import pytest
from Registry.Registry import RegSZ

from regrippy.plugins.printer_ports import Plugin

from .reg_mock import (LoggerMock, RegistryKeyMock, RegistryMock,
                       RegistryValueMock)

PORTS = [
    "COM1:",
    "COM2:",
    "FILE1:",
    "LPT1:",
    "LPT2:",
    "Ne00:",
    "Ne01:",
    "nul:",
    "PORTPROMPT:",
    "C:\\Windows\\System32\\hello.dll",
]


@pytest.fixture
def mock_reg():
    key = RegistryKeyMock.build(r"Microsoft\Windows NT\CurrentVersion\Ports")
    reg = RegistryMock("SOFTWARE", "software", key.root())

    for v in PORTS:
        val = RegistryValueMock(v, "", RegSZ)
        key.add_value(val)

    return reg


def test_printer_ports(mock_reg):
    p = Plugin(mock_reg, LoggerMock(), "SOFTWARE", "-")

    results = list(p.run())

    assert len(results) == len(PORTS), f"There should be {len(PORTS)} results"

    for r in results:
        assert r.value_name in PORTS, "The printer port should be valid"



from regrippy import BasePlugin, PluginResult, mactime


class Plugin(BasePlugin):
    """Lists all network redirections configured on the system"""

    __REGHIVE__ = "SYSTEM"

    def run(self):
        ccs = self.get_currentcontrolset_path()
        if not ccs:
            return

        for proto in ["tcp", "udp"]:
            key = self.open_key(ccs + "\\Services\\PortProxy\\v4tov4\\" + proto)
            if not key:
                continue

            for value in key.values():
                res = PluginResult(key=key, value=value)
                res.custom["proto"] = proto
                yield res

    def display_human(self, result):
        print(f"{result.custom['proto']} - {result.value_name} => {result.value_data}")

    def display_mactime(self, result):
        print(
            mactime(
                name=f"{result.custom['proto']} {result.value_name} {result.value_data}",
                mtime=result.mtime,
            )
        )
